@@ -1,17 +1,17 @@
 /**********************************************************************
 atom.h - Handle OBAtom class.
- 
+
 Copyright (C) 1998-2001 by OpenEye Scientific Software, Inc.
 Some portions Copyright (C) 2001-2006 by Geoffrey R. Hutchison
 Some portions Copyright (C) 2003 by Michael Banck
- 
+
 This file is part of the Open Babel project.
-For more information, see <http://openbabel.sourceforge.net/>
- 
+For more information, see <http://openbabel.org/>
+
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation version 2 of the License.
- 
+
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -32,6 +32,7 @@ GNU General Public License for more details.
 
 #include <openbabel/base.h>
 #include <openbabel/residue.h>
+#include <openbabel/math/vector3.h>
 
 namespace OpenBabel
 {
@@ -69,16 +70,17 @@ namespace OpenBabel
 #define OB_POS_CHIRAL_ATOM (1<<10)
   //! Atom has - chiral volume
 #define OB_NEG_CHIRAL_ATOM (1<<11)
-  //! Atom has no hydrogen attached. Temporary use only during SMILES input
+  //! Atom has no hydrogen attached. Temporary use only during input of some formats
 #define OB_ATOM_HAS_NO_H   (1<<12)
-  // 13-16 currently unused
+  //! Atom is not hydrogen deficient. (for SMILES input)
+#define OB_ATOM_NOT_H_DEFICIENT (1<<13)
 
   // Class OBAtom
   // class introduction in atom.cpp
  class OBAPI OBAtom: public OBBase
     {
     protected:
-      char                          _ele;       //!< atomic number (type char to minimize space -- allows for 0..255 elements)
+      unsigned char                 _ele;       //!< atomic number (type unsigned char to minimize space -- allows for 0..255 elements)
       char                          _impval;    //!< implicit valence
       char                          _type[6];   //!< atomic type
       short                         _fcharge;   //!< formal charge
@@ -97,6 +99,8 @@ namespace OpenBabel
       mutable vector3               _v;         //!< coordinate vector
       OBResidue                    *_residue;   //!< parent residue (if applicable)
 
+      unsigned long                 _id;        //!< unique id
+
       //! \return All flags
       int  GetFlag() const    {  return(_flags);  }
       //! Sets the bitwise @p flag
@@ -105,6 +109,11 @@ namespace OpenBabel
       bool HasFlag(int flag)  {  return((_flags & flag) ? true : false); }
 
     public:
+      enum StereoFlag {
+
+      };
+
+
        //! Used internally by graph traversal algorithms
       bool Visit;
 
@@ -114,6 +123,11 @@ namespace OpenBabel
       virtual ~OBAtom();
       //! Assignment
       OBAtom &operator = (OBAtom &);
+      //! Equivalence
+      bool operator==(const OBAtom * other) const {  return (GetIdx() == other->GetIdx()); }
+      //! Duplicate another atom. Copies all information with the exception of index
+      //! \since version 2.2
+      void Duplicate(OBAtom *);
       //! Clear all data. Calls OBBase::Clear() to handle any generic data.
       //! \return True if successful.
       bool Clear();
@@ -122,6 +136,7 @@ namespace OpenBabel
       //@{
       //! Set atom index (i.e., in an OBMol)
       void SetIdx(int idx)    { _idx = idx; _cidx = (idx-1)*3; }
+      void SetId(unsigned long id) { _id = id; }
       //! Set atom hybridization (i.e., 1 = sp, 2 = sp2, 3 = sp3 ...)
       void SetHyb(int hyb)    { _hyb = hyb; }
       //! Set atomic number
@@ -139,9 +154,9 @@ namespace OpenBabel
       //! Set the atomic spin to @p spin. See _spinmultiplicity
       void SetSpinMultiplicity(short spin){ _spinmultiplicity = spin; }
       //! Set the atomic type symbol (see OBTypeTable and OBAtomTyper for more)
-      void SetType(char *type);
+      void SetType(const char *type);
       //! Set the atomic type symbol (see OBTypeTable and OBAtomTyper for more)
-      void SetType(std::string &type);
+      void SetType(const std::string &type);
       //! Set the partial charge to @p pcharge
       void SetPartialCharge(double pcharge){ _pcharge = pcharge; }
       //! Set the coordinate vector for this atom to @p v as a vector3
@@ -160,15 +175,15 @@ namespace OpenBabel
       void SetAromatic()                  { SetFlag(OB_AROMATIC_ATOM); }
       //! Clear aromatic information from the atom
       void UnsetAromatic()                { _flags &= (~(OB_AROMATIC_ATOM)); }
-      //! Mark atom as having SMILES clockwise stereochemistry (i.e., "@@")
+      //! \deprecated
       void SetClockwiseStereo()           { SetFlag(OB_CSTEREO_ATOM|OB_CHIRAL_ATOM); }
-      //! Mark atom as having SMILES anticlockwise stereochemistry (i.e., "@")
+      //! \deprecated
       void SetAntiClockwiseStereo()       { SetFlag(OB_ACSTEREO_ATOM|OB_CHIRAL_ATOM); }
-      //! Mark an atom as having + chiral volume
+      //! \deprecated
       void SetPositiveStereo() { SetFlag(OB_POS_CHIRAL_ATOM|OB_CHIRAL_ATOM); }
-      //! Mark an atom as having - chiral volume
+      //! \deprecated
       void SetNegativeStereo() { SetFlag(OB_NEG_CHIRAL_ATOM|OB_CHIRAL_ATOM); }
-      //! Clear all stereochemistry information
+      //! \deprecated
       void UnsetStereo()
         {
           _flags &= ~(OB_ACSTEREO_ATOM);
@@ -193,7 +208,7 @@ namespace OpenBabel
       unsigned int GetAtomicNum()     const { return((unsigned int)_ele); }
       //! \return the isotope for this atom, if specified, or 0 for unspecified
       unsigned short int GetIsotope() const { return(_isotope);    }
-      //! \return the atomic spin, e.g., 0 (default) for singlet,
+      //! \return the atomic spin, e.g., 0 (default) for normal atoms - note that this value is a convention,
       //!   2 for radical  1 or 3 for carbene
       int          GetSpinMultiplicity() const { return(_spinmultiplicity); }
       //! \return the atomic mass of this atom given by standard IUPAC
@@ -204,6 +219,8 @@ namespace OpenBabel
       double     GetExactMass()     const;
       //! \return the internal atom index (e.g., inside an OBMol)
       unsigned int GetIdx()           const { return((int)_idx);  }
+      unsigned int GetIndex() const { return _idx - 1; }
+      unsigned long GetId() const { return _id; }
       //! \return the index into a pointer-driven array as used by
       //!   GetCoordPtr() or SetCoordPtr()
       unsigned int GetCoordinateIdx() const { return((int)_cidx); }
@@ -212,9 +229,9 @@ namespace OpenBabel
       //! \return The current number of explicit connections
       unsigned int GetValence()       const
         {
-          return((_vbond.empty()) ? 0 : _vbond.size());
+          return((_vbond.empty()) ? 0 : static_cast<unsigned int> (_vbond.size()));
         }
-      //! \return The hybridization of this atom (i.e. 1 for sp, 2 for sp2, 3 for sp3)
+      //! \return The hybridization of this atom: 1 for sp, 2 for sp2, 3 for sp3, 4 for sq. planar, 5 for trig. bipy, 6 for octahedral
       unsigned int GetHyb()             const;
       //! \return The implicit valence of this atom type (i.e. maximum number of connections expected)
       unsigned int GetImplicitValence() const;
@@ -226,26 +243,26 @@ namespace OpenBabel
       char        *GetType();
 
       //! \return the x coordinate
-      double      GetX()    {        return(x());    }
+      double      GetX() const   {        return(x());    }
       //! \return the y coordinate
-      double      GetY()    {        return(y());    }
+      double      GetY() const  {        return(y());    }
       //! \return the z coordinate
-      double      GetZ()    {        return(z());    }
+      double      GetZ() const  {        return(z());    }
 
       // These methods check to see if there is a coordinate pointer
       // or an internal vector (e.g., SetCoordPtr())
       //! \return the x coordinate
-      double      x() {
+      double      x() const {
         if (_c)            return((*_c)[_cidx]);
         else               return _v.x();
       }
       //! \return the y coordinate
-      double      y() {
+      double      y() const {
         if (_c)            return((*_c)[_cidx+1]);
         else               return _v.y();
       }
       //! \return the z coordinate
-      double      z() {
+      double      z() const {
         if (_c)            return((*_c)[_cidx+2]);
         else               return _v.z();
       }
@@ -259,11 +276,15 @@ namespace OpenBabel
       }
       //! \return the coordinates as a vector3 object
       vector3   &GetVector();
+      //! \return the coordinates as a vector3 object
       const vector3   &GetVector() const;
       //! \return the partial charge of this atom, calculating a Gasteiger charge if needed
       double     GetPartialCharge();
       //! \return the residue which contains this atom, or NULL if none exists
       OBResidue *GetResidue();
+      //! \param perception implies whether chain perception should occur
+      //! \return the residue which contains this atom, or NULL if none exists
+      OBResidue *GetResidue(bool perception);
       //! \return the molecule which contains this atom, or NULL if none exists
       OBMol     *GetParent()        {return((OBMol*)_parent);}
       //! Create a vector for a new bond from this atom, with length given by the supplied parameter
@@ -355,6 +376,13 @@ namespace OpenBabel
       void ForceNoH() {SetFlag(OB_ATOM_HAS_NO_H);}
       //! \return if atom has been marked as having no hydrogens attached
       bool HasNoHForced() {return HasFlag(OB_ATOM_HAS_NO_H);}
+
+      //! Mark that atom is not hydrogen deficient (For SMILES input)
+      //! \since version 2.2
+      void ForceImplH() {SetFlag(OB_ATOM_NOT_H_DEFICIENT);}
+      //! \return if atom has been marked as having no hydrogens attached
+      //! \since version 2.2
+      bool HasImplHForced() {return HasFlag(OB_ATOM_NOT_H_DEFICIENT);}
       //@}
 
       //! \name Property information
@@ -384,6 +412,7 @@ namespace OpenBabel
       bool HasResidue()    { return(_residue != NULL);    }
       //! \return Is the atom hydrogen?
       bool IsHydrogen()    { return(GetAtomicNum() == 1); }
+      bool IsHydrogen() const { return(GetAtomicNum() == 1); }
       //! \return Is the atom carbon?
       bool IsCarbon()      { return(GetAtomicNum() == 6); }
       //! \return Is the atom nitrogen?
@@ -407,7 +436,7 @@ namespace OpenBabel
       bool IsNotCorH();
       //! \return Is this atom directly connected to the supplied OBAtom?
       bool IsConnected(OBAtom*);
-      //! \return Is this atom related to the supplied OBAtom in 
+      //! \return Is this atom related to the supplied OBAtom in
       //!  a 1,3 bonding pattern?
       bool IsOneThree(OBAtom*);
       //! \return Is this atom related to the supplied OBAtom in
@@ -415,7 +444,7 @@ namespace OpenBabel
       bool IsOneFour(OBAtom*);
       //! \return Is this atom an oxygen in a carboxyl (-CO2 or CO2H) group?
       bool IsCarboxylOxygen();
-      //! \return Is this atom an oxygen in a phosphate (R-PO3) group? 
+      //! \return Is this atom an oxygen in a phosphate (R-PO3) group?
       bool IsPhosphateOxygen();
       //! \return Is this atom an oxygen in a sulfate (-SO3) group?
       bool IsSulfateOxygen();
@@ -436,20 +465,18 @@ namespace OpenBabel
       bool IsChiral();
       //! \return Is this atom an axial atom in a ring
       bool IsAxial();
-      //! \return Does this atom have SMILES-specified clockwise "@@" stereochemistry?
+      //! \deprecated
       bool IsClockwise()         { return(HasFlag(OB_CSTEREO_ATOM));  }
-      //! \return Does this atom have SMILES-specified anticlockwise "@" stereochemistry?
+      //! \deprecated
       bool IsAntiClockwise()     { return(HasFlag(OB_ACSTEREO_ATOM)); }
-      //! \return Does this atom have a positive chiral volume?
+      //! \deprecated
       bool IsPositiveStereo() { return(HasFlag(OB_POS_CHIRAL_ATOM)); }
-      //! \return Does this atom have a negative chiral volume?
+      //! \deprecated
       bool IsNegativeStereo() { return(HasFlag(OB_NEG_CHIRAL_ATOM)); }
-      //! \return Does this atom have SMILES-specified stereochemistry?
-      bool HasChiralitySpecified()
-        { return(HasFlag(OB_CSTEREO_ATOM|OB_ACSTEREO_ATOM)); }
-      //! \return Does this atom have a specified chiral volume?
-      bool HasChiralVolume()
-        { return(HasFlag(OB_POS_CHIRAL_ATOM|OB_NEG_CHIRAL_ATOM)); }
+      //! \deprecated
+      bool HasChiralitySpecified() { return(HasFlag(OB_CSTEREO_ATOM|OB_ACSTEREO_ATOM)); }
+      //! \deprecated
+      bool HasChiralVolume() { return(HasFlag(OB_POS_CHIRAL_ATOM|OB_NEG_CHIRAL_ATOM)); }
       //! \return Is this atom a hydrogen-bond acceptor (receptor)?
       bool IsHbondAcceptor();
       //! \return Is this atom a hydrogen-bond donor?
