@@ -126,6 +126,9 @@ namespace OpenBabel
     bondLength += etab.CorrectedBondRad(atom1->GetAtomicNum(), atom1->GetHyb());
     bondLength += etab.CorrectedBondRad(atom2->GetAtomicNum(), atom2->GetHyb());
 
+    if (bondLength < 1.0)
+      bondLength = 1.0;
+
     // These are based on OBBond::GetEquibLength
     if (bondOrder == -1) // aromatic
       bondLength *= 0.93;
@@ -144,7 +147,7 @@ namespace OpenBabel
 
   vector3 OBBuilder::GetNewBondVector(OBAtom *atom, double length)
   {
-    vector3 bond1, bond2, bond3, bond4, bond5, v1, v2, newbond;
+    vector3 bond1, bond2, bond3, bond4, bond5, v1, v2, v3, newbond;
 
     bond1 = VZero;
     bond2 = VZero;
@@ -220,13 +223,13 @@ namespace OpenBabel
             angle = fabs(acos(dot(bond1, vrand)) * RAD_TO_DEG);
           }
           // there is no a-2 atom
-          v1 = cross(bond1, vrand);
+          v1 = cross(bond1, vrand); // so find a perpendicular, given the random vector (this doesn't matter here)
           v2 = cross(bond1, v1);
         } else {
           v1 = cross(bond1, bond2);
           v2 = cross(bond1, v1);
-          v2 = v2.normalize();
         }
+        v2 = v2.normalize();
 
         // check to see if atom is a square planar in disguise
         if (atom->GetHyb() == 3) {
@@ -236,11 +239,11 @@ namespace OpenBabel
         }
 
         if (atom->GetHyb() == 1)
-          newbond = bond1;
+          newbond = bond1; // i.e., in the exact opposite direction
         else if (atom->GetHyb() == 2)
           newbond = bond1 - v2 * tan(DEG_TO_RAD*60.0);
         else if (atom->GetHyb() == 3)
-          newbond = bond1 - v2 * tan(DEG_TO_RAD*70.5);
+          newbond = bond1 - v2 * tan(DEG_TO_RAD*(180.0 - 109.471));
         else if (atom->GetHyb() == 4)
           newbond = bond1; // like 5-coordinate below, we want a 180-degree bond (trans)
         else if (atom->GetHyb() == 5) {
@@ -277,9 +280,10 @@ namespace OpenBabel
         if (atom->GetHyb() == 2)
           newbond = v1;
         if (atom->GetHyb() == 3) {
-          v2 = cross(bond1, bond2);
-          //v1 = v1.normalize();
-          newbond = v2 + v1 * tan(DEG_TO_RAD*35.25);
+          v2 = cross(bond1, bond2); // find the perpendicular
+          v2.normalize();
+          newbond = bond1 - v2 * tan(DEG_TO_RAD*(180.0 - 109.471));
+          newbond = v2 + v1 * (sqrt(2.0) / 2.0); // used to be tan(70.53 degrees/2) which is sqrt(2.0) / 2.0
         }
         if (atom->GetHyb() == 5 || atom->GetHyb() == 4) {
           /* add the first equatorial atom, orthogonally to bond1 (and bond2 = -bond1) */
@@ -672,6 +676,15 @@ namespace OpenBabel
     if (fragment == GetFragment(a))
       return false; // a and b are in the same fragment
 
+    bool connectedFrag = true; // normal case
+    // If we don't have any neighbors, assume that the fragment
+    // is inserted at the end of the molecule and set anything after the atom.
+    // This lets us place fragments like Cp rings with dummy atoms
+    if (b->GetAtomicNum() == 0) {
+      connectedFrag = false;
+      fragment.SetRangeOn(b->GetIdx(), mol.NumAtoms());
+    }
+
     vector3 posa = a->GetVector();
     vector3 posb = b->GetVector();
     //
@@ -693,6 +706,30 @@ namespace OpenBabel
     double xyang, yzang, xzang;
 
     vector3 fragdir = GetNewBondVector(b); // b is at origin
+    vector3 crossdir;
+    if (!connectedFrag) { // nothing bonded to b, like a Cp ring
+      vector3 firstDir, secondDir;
+      // Try finding the next atom
+      OBAtom *nextAtom = mol.GetAtom(b->GetIdx() + 1);
+      if (nextAtom) {
+        firstDir = nextAtom->GetVector() - b->GetVector();
+        // we'll try finding another atom
+        OBAtom *secondAtom = mol.GetAtom(b->GetIdx() + 2);
+        if (secondAtom)
+          secondDir = secondAtom->GetVector() - b->GetVector();
+        else
+          secondDir.randomUnitVector(); // pick something at random
+        // but not too shallow, or the cross product won't work well
+        double angle = fabs(acos(dot(firstDir, secondDir)) * RAD_TO_DEG);
+        while (angle < 45.0 || angle > 135.0) {
+          secondDir.randomUnitVector();
+          angle = fabs(acos(dot(firstDir, secondDir)) * RAD_TO_DEG);
+        }
+        // Now we find a perpendicular vector to the fragment
+        crossdir = cross(firstDir, secondDir);
+        fragdir = crossdir;
+      }
+    }
     xyang = vectorAngle(vector3(moldir.x(), moldir.y(), 0.0), vector3(fragdir.x(), fragdir.y(), 0.0));
     if (cross(vector3(moldir.x(), moldir.y(), 0.0), vector3(fragdir.x(), fragdir.y(), 0.0)).z() > 0) {
       xyang = 180 + xyang;
@@ -711,6 +748,8 @@ namespace OpenBabel
     }
 
     fragdir = GetNewBondVector(b);
+    if (!connectedFrag)
+      fragdir = crossdir;
     xzang = vectorAngle(vector3(moldir.x(), moldir.z(), 0.0), vector3(fragdir.x(), fragdir.z(), 0.0));
     if (cross(vector3(moldir.x(), moldir.z(), 0.0), vector3(fragdir.x(), fragdir.z(), 0.0)).z() > 0) {
       xzang = 180 - xzang;
@@ -729,6 +768,8 @@ namespace OpenBabel
     }
 
     fragdir = GetNewBondVector(b);
+    if (!connectedFrag)
+      fragdir = crossdir;
     yzang = vectorAngle(vector3(moldir.y(), moldir.z(), 0.0), vector3(fragdir.y(), fragdir.z(), 0.0));
     if (cross(vector3(moldir.y(), moldir.z(), 0.0), vector3(fragdir.y(), fragdir.z(), 0.0)).z() > 0) {
       yzang = 180 + yzang;
@@ -1144,7 +1185,7 @@ namespace OpenBabel
     // correct the chirality
     CorrectStereoBonds(workMol);
     CorrectStereoAtoms(workMol);
-
+    workMol.SetChiralityPerceived();
     mol = workMol;
     mol.SetDimension(3);
 
