@@ -1,16 +1,16 @@
 /**********************************************************************
 data.cpp - Global data and resource file parsers.
- 
+
 Copyright (C) 1998-2001 by OpenEye Scientific Software, Inc.
 Some portions Copyright (C) 2001-2008 by Geoffrey R. Hutchison
- 
+
 This file is part of the Open Babel project.
-For more information, see <http://openbabel.sourceforge.net/>
- 
+For more information, see <http://openbabel.org/>
+
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation version 2 of the License.
- 
+
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -49,7 +49,7 @@ namespace OpenBabel
 
   /** \class OBElementTable data.h <openbabel/data.h>
       \brief Periodic Table of the Elements
- 
+
       Translating element data is a common task given that many file
       formats give either element symbol or atomic number information, but
       not both. The OBElementTable class facilitates conversion between
@@ -63,7 +63,7 @@ namespace OpenBabel
       cout << "The atomic number for Sulfur is " << etab.GetAtomicNum(16) << endl;
       cout << "The van der Waal radius for Nitrogen is " << etab.GetVdwRad(7);
       \endcode
- 
+
       Stored information in the OBElementTable includes elemental:
       - symbols
       - covalent radii
@@ -127,10 +127,10 @@ namespace OpenBabel
   }
 
   unsigned int OBElementTable::GetNumberOfElements()
-  { 
+  {
     if (!_init)
       Init();
-    
+
     return _element.size();
   }
 
@@ -201,7 +201,7 @@ namespace OpenBabel
 
     return(_element[atomicnum]->GetElectronAffinity());
   }
- 
+
   vector<double> OBElementTable::GetRGB(int atomicnum)
   {
     if (!_init)
@@ -209,7 +209,7 @@ namespace OpenBabel
 
     vector <double> colors;
     colors.reserve(3);
-    
+
     if (atomicnum < 0 || atomicnum >= static_cast<int>(_element.size()))
       {
         colors.push_back(0.0);
@@ -224,7 +224,7 @@ namespace OpenBabel
 
     return (colors);
   }
- 
+
   string OBElementTable::GetName(int atomicnum)
   {
     if (!_init)
@@ -313,30 +313,58 @@ namespace OpenBabel
     return GetAtomicNum(sym, temp);
   }
 
-  int OBElementTable::GetAtomicNum(const char *sym, int &iso)
+  int OBElementTable::GetAtomicNum(const char *identifier, int &iso)
   {
     if (!_init)
       Init();
 
+    char buffer[BUFF_SIZE]; // error buffer
+
+    // Compare to symbol
     vector<OBElement*>::iterator i;
     for (i = _element.begin();i != _element.end();++i)
-      if (!strncasecmp(sym,(*i)->GetSymbol(),3))
+      if (!strncasecmp(identifier,(*i)->GetSymbol(),3))
         return((*i)->GetAtomicNum());
-    if (strcasecmp(sym, "D") == 0)
+
+    // Compare to IUPAC name
+    for (i = _element.begin();i != _element.end();++i)
+      if (strncasecmp(identifier,(*i)->GetName().c_str(),5) == 0)
+        return((*i)->GetAtomicNum());
+
+    if (strcasecmp(identifier, "D") == 0 ||
+        (strcasecmp(identifier, "Deuterium") == 0) )
       {
         iso = 2;
         return(1);
       }
-    else if (strcasecmp(sym, "T") == 0)
+    else if (strcasecmp(identifier, "T") == 0 ||
+             (strcasecmp(identifier, "Tritium") == 0) )
       {
         iso = 3;
         return(1);
       }
+    else if (strcasecmp(identifier, "Hl") == 0) // ligand hydrogen -- found in some CIF PR#3048959.
+      {
+        snprintf(buffer, BUFF_SIZE, "Cannot understand the element label %s. Guessing it's hydrogen.", identifier);
+        obErrorLog.ThrowError(__FUNCTION__, buffer, obWarning);
+        return(1);
+      }
     else
       iso = 0;
+
+    if(identifier[0]!='*')
+      {
+        snprintf(buffer, BUFF_SIZE, "Cannot understand the element label %s.", identifier);
+        obErrorLog.ThrowError(__FUNCTION__, buffer, obWarning);
+      }
     return(0);
   }
 
+  int OBElementTable::GetAtomicNum(string name, int &iso)
+  {
+    return GetAtomicNum(name.c_str(), iso);
+  }
+
   /** \class OBIsotopeTable data.h <openbabel/data.h>
       \brief Table of atomic isotope masses
   */
@@ -387,6 +415,8 @@ namespace OpenBabel
 
     if (ele > _isotopes.size())
       return 0.0;
+    if (_isotopes[ele].size() == 0)
+      return 0.0; // PR#2996661
 
     unsigned int iso;
     for (iso = 0; iso < _isotopes[ele].size(); ++iso)
@@ -398,7 +428,7 @@ namespace OpenBabel
 
   /** \class OBTypeTable data.h <openbabel/data.h>
       \brief Atom Type Translation Table
- 
+
       Molecular file formats frequently store information about atoms in an
       atom type field. Some formats store only the element for each atom,
       while others include hybridization and local environments, such as the
@@ -410,7 +440,7 @@ namespace OpenBabel
       extern OBTypeTable ttab in mol.h.  The following code demonstrates how
       to use the OBTypeTable class to translate the internal representation
       of atom types in an OBMol Internal to Sybyl Mol2 atom types.
- 
+
       \code
       ttab.SetFromType("INT");
       ttab.SetToType("SYB");
@@ -424,7 +454,7 @@ namespace OpenBabel
          cout << "atom number " << atom->GetIdx() << "has mol2 type " << dst << endl;
       }
       \endcode
- 
+
       Current atom types include (defined in the top line of the data file types.txt):
       - INT (Open Babel internal codes)
       - ATN (atomic numbers)
@@ -681,6 +711,8 @@ namespace OpenBabel
               break;
             if (r1->GetName() != r2->GetName())
               break;
+            if (r1->GetChain() != r2->GetChain())
+              break; // Fixes PR#2889763 - Fabian
 
             if ((bo = LookupBO(r1->GetAtomID(a1),r2->GetAtomID(a2))))
               {
@@ -728,11 +760,12 @@ namespace OpenBabel
                 // Leave the protonation/deprotonation to phmodel.txt
                 a1->SetType("O3");
                 a1->SetHyb(3);
-                a1->SetFormalCharge(0);
+                // PR#3203039 -- Fix from Magnus Lundborg
+                //                a1->SetFormalCharge(0);
               }
             continue;
           }
-	
+
         r1 = a1->GetResidue();
         if (r1 == NULL) continue; // atoms may not have residues
         if (skipres.length() && r1->GetNumString() == skipres)
@@ -880,7 +913,7 @@ namespace OpenBabel
     // Check return value from OpenDatafile
     // Suggestion from Zhiguo Liu
     string fn_open = OpenDatafile(ifs, _filename, _envvar);
-     
+
     if (fn_open != "" && (ifs))
       {
         while(ifs.getline(charBuffer,BUFF_SIZE))
@@ -891,6 +924,8 @@ namespace OpenBabel
       // If all else fails, use the compiled in values
       if (_dataptr)
         {
+          obErrorLog.ThrowError(__FUNCTION__, "Cannot open " + _filename + " defaulting to compiled data.", obWarning);
+
           const char *p1,*p2;
           for (p1 = p2 = _dataptr;*p2 != '\0';++p2)
             if (*p2 == '\n')
@@ -911,7 +946,7 @@ namespace OpenBabel
 
     // return the locale to the original one
     obLocale.RestoreLocale();
-    
+
     if (ifs)
       ifs.close();
 
@@ -922,7 +957,7 @@ namespace OpenBabel
         s += "' which may cause further errors.";
         obErrorLog.ThrowError(__FUNCTION__, "Cannot initialize database", obWarning);
       }
-      
+
   }
 
 } // end namespace OpenBabel
