@@ -19,11 +19,15 @@ GNU General Public License for more details.
 
 #include <openbabel/mol.h>
 #include <openbabel/alias.h>
+#include <openbabel/atomclass.h>
 #include <openbabel/depict/depict.h>
 #include <openbabel/depict/painter.h>
 #include <algorithm> // std::reverse
 #include <iterator> // std::istream_iterator
 #include <openbabel/stereo/stereo.h>
+#include <openbabel/obiter.h>
+
+#include <cmath>
 
 #include <iostream>
 using namespace std;
@@ -32,15 +36,10 @@ namespace OpenBabel
 {
 
   enum {
-    TopLeft = 1,
-    TopCenter = 2,
-    TopRight = 3,
-    CenterLeft = 4,
-    Center = 5,
-    CenterRight = 6,
-    BottomLeft = 7,
-    BottomCenter = 8,
-    BottomRight = 9
+    Left,
+    Right,
+    Up,
+    Down
   };
 
   class OBDepictPrivate
@@ -53,6 +52,7 @@ namespace OpenBabel
       void DrawSimpleBond(OBAtom *beginAtom, OBAtom *endAtom, int order);
       void DrawWedge(OBAtom *beginAtom, OBAtom *endAtom);
       void DrawHash(OBAtom *beginAtom, OBAtom *endAtom);
+      void DrawWobblyBond(OBAtom *beginAtom, OBAtom *endAtom);
       void DrawRingBond(OBAtom *beginAtom, OBAtom *endAtom, const vector3 &center, int order);
       void DrawAtomLabel(const std::string &label, int alignment, const vector3 &pos);
 
@@ -78,6 +78,13 @@ namespace OpenBabel
     d->painter = painter;
   }
 
+  OBDepict::~OBDepict()
+  {
+    delete d->mol;
+    d->mol = NULL;
+    delete d;
+  }
+
   void OBDepict::SetBondLength(double length) 
   { 
     d->bondLength = length; 
@@ -194,21 +201,16 @@ namespace OpenBabel
     
     const double bias = -0.1; //towards left-alignment, which is more natural
     int alignment = 0;
-    if (direction.y() < 0.0) {
-      if (direction.x() < bias)
-        alignment = BottomRight;
-      else 
-        alignment = BottomLeft;
-    } else if (direction.x() > 0.0) {
-      if (direction.x() < bias)
-        alignment = TopRight;
+    if ((atom->GetValence() == 2) && (abs(direction.y()) > abs(direction.x()))) {
+      if (direction.y() <= 0.0)
+        alignment = Up;
       else
-        alignment = TopLeft;
+        alignment = Down;
     } else {
       if (direction.x() < bias)
-         alignment = CenterRight;
-      else 
-        alignment = CenterLeft;
+        alignment = Right;
+      else
+        alignment = Left;
     }
 
     return alignment;
@@ -275,7 +277,13 @@ namespace OpenBabel
     if (!d->painter)
       return false;
 
-    d->mol = mol;
+    if (d->mol != NULL)
+      delete d->mol;
+    d->mol = new OBMol(*mol); // Copy it
+    
+    OBAtomClassData* pac = NULL;
+    if(mol->HasData("Atom Class"))
+      pac = static_cast<OBAtomClassData*>(mol->GetData("Atom Class"));
 
     double width=0.0, height=0.0;
 
@@ -283,23 +291,30 @@ namespace OpenBabel
     OBBondIterator j;
     OBAtomIterator i;
 
+    
+    // Determine which should be wedge and hash bonds...
+    // Note: we need to do this before we invert the y-coordinate for depiction
+    std::map<OBBond*, enum OBStereo::BondDirection> updown;
+    std::map<OBBond*, OBStereo::Ref> from;
+    TetStereoToWedgeHash(*d->mol, updown, from);
+
     if(mol->NumAtoms()>0) {
-      // scale bond lengths
+      // scale bond lengths and invert the y coordinate (both SVG and Cairo use top left as the origin)
       double bondLengthSum = 0.0;
       for (OBBond *bond = mol->BeginBond(j); bond; bond = mol->NextBond(j))
         bondLengthSum += bond->GetLength();
       const double averageBondLength = bondLengthSum / mol->NumBonds();
       const double f = mol->NumBonds() ? d->bondLength / averageBondLength : 1.0;
-      for (atom = mol->BeginAtom(i); atom; atom = mol->NextAtom(i))
-        atom->SetVector(atom->GetX() * f, atom->GetY() * f, 0.0);
+      for (atom = d->mol->BeginAtom(i); atom; atom = d->mol->NextAtom(i))
+        atom->SetVector(atom->GetX() * f, - atom->GetY() * f, 0.0);
 
       // find min/max values
       double min_x, max_x;
       double min_y, max_y;
-      atom = mol->BeginAtom(i);
+      atom = d->mol->BeginAtom(i);
       min_x = max_x = atom->GetX();
       min_y = max_y = atom->GetY();
-      for (atom = mol->NextAtom(i); atom; atom = mol->NextAtom(i)) {
+      for (atom = d->mol->NextAtom(i); atom; atom = d->mol->NextAtom(i)) {
         min_x = std::min(min_x, atom->GetX());
         max_x = std::max(max_x, atom->GetX());
         min_y = std::min(min_y, atom->GetY());
@@ -308,7 +323,7 @@ namespace OpenBabel
 
       const double margin = 40.0;
       // translate all atoms so the bottom-left atom is at margin,margin
-      for (atom = mol->BeginAtom(i); atom; atom = mol->NextAtom(i))
+      for (atom = d->mol->BeginAtom(i); atom; atom = d->mol->NextAtom(i))
         atom->SetVector(atom->GetX() - min_x + margin, atom->GetY() - min_y + margin, 0.0);
 
       width  = max_x - min_x + 2*margin;
@@ -322,26 +337,31 @@ namespace OpenBabel
     d->painter->NewCanvas(width, height);
     
     // draw bonds
-    if(d->options & genWedgeHash)
-      d->SetWedgeAndHash(mol);
-    for (OBBond *bond = mol->BeginBond(j); bond; bond = mol->NextBond(j)) {
+    for (OBBond *bond = d->mol->BeginBond(j); bond; bond = d->mol->NextBond(j)) {
       OBAtom *begin = bond->GetBeginAtom();
       OBAtom *end = bond->GetEndAtom();
-
       if((d->options & internalColor) && bond->HasData("color"))
         d->painter->SetPenColor(OBColor(bond->GetData("color")->GetValue()));
       else
         d->painter->SetPenColor(d->bondColor);
 
-      if (bond->IsWedge()) {
-        d->DrawWedge(begin, end);
-      } else if (bond->IsHash()) {
-        d->DrawHash(begin, end);
-      } else if (!bond->IsInRing()) {
-        d->DrawSimpleBond(begin, end, bond->GetBO());
+      if(from.find(bond)!=from.end()) {
+        //is a wedge or hash bond
+        if(from[bond]==bond->GetEndAtom()->GetId())
+          swap(begin, end);
+        if(updown[bond]==OBStereo::UpBond)
+          d->DrawWedge(begin, end);
+        else if(updown[bond]==OBStereo::DownBond)
+          d->DrawHash(begin, end);
+        else {
+          //This is a bond to a chiral center specified as unknown
+          d->DrawWobblyBond(begin, end);
+        }
       }
+      else if (!bond->IsInRing())
+        d->DrawSimpleBond(begin, end, bond->GetBO());
     }
-    
+
     // draw ring bonds
     std::vector<OBRing*> rings(mol->GetSSSR());
     OBBitVec drawnBonds;
@@ -350,19 +370,19 @@ namespace OpenBabel
       std::vector<int> indexes = ring->_path;
       vector3 center(VZero);
       for (std::vector<int>::iterator l = indexes.begin(); l != indexes.end(); ++l) {
-        center += mol->GetAtom(*l)->GetVector();        
+        center += d->mol->GetAtom(*l)->GetVector();        
       }
       center /= indexes.size();
 
       for (unsigned int l = 0; l < indexes.size(); ++l) {
-        OBAtom *begin = mol->GetAtom(indexes[l]);
+        OBAtom *begin = d->mol->GetAtom(indexes[l]);
         OBAtom *end;
         if (l+1 < indexes.size())
-          end = mol->GetAtom(indexes[l+1]);
+          end = d->mol->GetAtom(indexes[l+1]);
         else
-          end = mol->GetAtom(indexes[0]);
+          end = d->mol->GetAtom(indexes[0]);
 
-        OBBond *ringBond = mol->GetBond(begin, end);
+        OBBond *ringBond = d->mol->GetBond(begin, end);
         if (drawnBonds.BitIsSet(ringBond->GetId()))
           continue;
 
@@ -378,16 +398,14 @@ namespace OpenBabel
     }
 
     // draw atom labels
-    for (atom = mol->BeginAtom(i); atom; atom = mol->NextAtom(i)) {
+    for (atom = d->mol->BeginAtom(i); atom; atom = d->mol->NextAtom(i)) {
       double x = atom->GetX();
       double y = atom->GetY();
 
       int alignment = GetLabelAlignment(atom);
       bool rightAligned = false;
       switch (alignment) {
-        case TopRight:
-        case CenterRight:
-        case BottomRight:
+        case Right:
           rightAligned = true;
         default:
           break;
@@ -405,15 +423,13 @@ namespace OpenBabel
       int spin = atom->GetSpinMultiplicity();
       if(charge || spin) {
         OBFontMetrics metrics = d->painter->GetFontMetrics("N");
-        double yoffset = d->HasLabel(atom) ? 0.4 * metrics.height : 0.0;
-        switch (GetLabelAlignment(atom)) {
-          case TopCenter:
-          case TopRight:
-          case TopLeft:
-          case CenterLeft:
-          case CenterRight:
+        double yoffset = d->HasLabel(atom) ? -0.2 * metrics.height : -0.2 * metrics.height;
+        /*switch (GetLabelAlignment(atom)) {
+          case Up:
+          case Left:
+          case Right:
             yoffset = - 1.2 * metrics.height;
-        }
+        }*/
         stringstream ss;
         if(charge) {
           if(abs(charge)!=1)
@@ -426,7 +442,7 @@ namespace OpenBabel
         }
         if(spin || charge<0)
           d->painter->SetFontSize(2 * metrics.fontSize);
-        d->painter->DrawText(x-0.4*metrics.width, y-yoffset, ss.str());
+        d->painter->DrawText(x + 0.4*metrics.width, y+yoffset, ss.str());
         d->painter->SetFontSize(metrics.fontSize);//restore
       }
  
@@ -452,6 +468,14 @@ namespace OpenBabel
         OBColor aliasColor = !ad->GetColor().empty() ? ad->GetColor() : d->bondColor; 
           d->painter->SetPenColor(aliasColor);
       }
+
+      //Atoms with no AliasData, but 0 atomic num and atomclass==n are output as Rn 
+      else if(pac && atom->GetAtomicNum()==0 && pac->HasClass(atom->GetIdx()))
+      {
+        ss << 'R' << pac->GetClass(atom->GetIdx());
+        d->painter->SetPenColor(OBColor("black"));
+      }
+
       else {
         const char* atomSymbol;
         if(atom->IsHydrogen() && atom->GetIsotope()>1)
@@ -479,6 +503,44 @@ namespace OpenBabel
     return true;
   }
 
+  void OBDepictPrivate::DrawWobblyBond(OBAtom *beginAtom, OBAtom *endAtom)
+  {
+    vector3 begin = beginAtom->GetVector();
+    vector3 end = endAtom->GetVector();
+    vector3 vb = end - begin;
+
+    if (HasLabel(beginAtom))
+      begin += 0.33 * vb;
+    if (HasLabel(endAtom))
+      end -= 0.33 * vb;
+    
+    vb = end - begin; // Resize the extents of the vb vector
+
+    vector3 orthogonalLine = cross(vb, VZ);
+    orthogonalLine.normalize();
+    orthogonalLine *= 0.5 * bondWidth;
+
+    double lines[6] = { 0.20, 0.36, 0.52, 0.68, 0.84, 1.0 };
+
+    // This code is adapted from DrawWedge():
+    // What we do is just join up the opposite ends of each of the wedge strokes
+    // to create a zig-zag bond
+
+    double oldx, oldy, newx, newy;
+    oldx = begin.x();
+    oldy = begin.y();
+    int sign = 1;
+    for (int k = 0; k < 6; ++k) {
+      double w = lines[k];
+      newx = begin.x() + vb.x() * w + sign * orthogonalLine.x() * w;
+      newy = begin.y() + vb.y() * w + sign * orthogonalLine.y() * w;
+      painter->DrawLine(oldx, oldy, newx, newy);
+      oldx = newx;
+      oldy = newy;
+      sign =- sign;
+    }
+  } 
+
   void OBDepictPrivate::DrawWedge(OBAtom *beginAtom, OBAtom *endAtom)
   {
     vector3 begin = beginAtom->GetVector();
@@ -495,11 +557,11 @@ namespace OpenBabel
     orthogonalLine *= 0.5 * bondWidth;
     std::vector<std::pair<double,double> > points;
 
-    points.push_back(std::pair<double,double>(end.x(), end.y()));
-    points.push_back(std::pair<double,double>(begin.x() + orthogonalLine.x(), 
-                                              begin.y() + orthogonalLine.y()));
-    points.push_back(std::pair<double,double>(begin.x() - orthogonalLine.x(), 
-                                              begin.y() - orthogonalLine.y()));
+    points.push_back(std::pair<double,double>(begin.x(), begin.y()));
+    points.push_back(std::pair<double,double>(end.x() + orthogonalLine.x(), 
+                                              end.y() + orthogonalLine.y()));
+    points.push_back(std::pair<double,double>(end.x() - orthogonalLine.x(), 
+                                              end.y() - orthogonalLine.y()));
     painter->DrawPolygon(points);
   }
 
@@ -510,22 +572,24 @@ namespace OpenBabel
     vector3 vb = end - begin;
 
     if (HasLabel(beginAtom))
-      begin += 0.26 * vb;
+      begin += 0.33 * vb;
     if (HasLabel(endAtom))
-      end -= 0.26 * vb;
+      end -= 0.33 * vb;
+    
+    vb = end - begin; // Resize the extents of the vb vector
 
     vector3 orthogonalLine = cross(vb, VZ);
     orthogonalLine.normalize();
     orthogonalLine *= 0.5 * bondWidth;
 
-    double lines[7] = { 0.10, 0.20, 0.30, 0.40, 0.50, 0.60, 0.70 };
+    double lines[6] = { 0.20, 0.36, 0.52, 0.68, 0.84, 1.0 };
 
-    for (int k = 0; k < 7; ++k) {
+    for (int k = 0; k < 6; ++k) {
       double w = lines[k];
-      painter->DrawLine(end.x() - vb.x() * w + orthogonalLine.x() * w, 
-                        end.y() - vb.y() * w + orthogonalLine.y() * w, 
-                        end.x() - vb.x() * w - orthogonalLine.x() * w, 
-                        end.y() - vb.y() * w - orthogonalLine.y() * w);
+      painter->DrawLine(begin.x() + vb.x() * w + orthogonalLine.x() * w, 
+                        begin.y() + vb.y() * w + orthogonalLine.y() * w, 
+                        begin.x() + vb.x() * w - orthogonalLine.x() * w, 
+                        begin.y() + vb.y() * w - orthogonalLine.y() * w);
     }
   } 
   
@@ -534,21 +598,58 @@ namespace OpenBabel
     vector3 begin = beginAtom->GetVector();
     vector3 end = endAtom->GetVector();
     vector3 vb = end - begin;
+    
+    vb.normalize();
 
     if (HasLabel(beginAtom))
-      begin += 0.26 * vb;
+      begin += 13. * vb; // Length is normally 40
     if (HasLabel(endAtom))
-      end -= 0.26 * vb;
+      end -= 13. * vb;
 
     if (order == 1) {
       painter->DrawLine(begin.x(), begin.y(), end.x(), end.y());
     } else if (order == 2) {
       vector3 orthogonalLine = cross(end - begin, VZ).normalize();
-      vector3 offset = orthogonalLine * 0.5 * bondSpacing;
-      painter->DrawLine(begin.x() + offset.x(), begin.y() + offset.y(), 
-                        end.x() + offset.x(), end.y() + offset.y());
-      painter->DrawLine(begin.x() - offset.x(), begin.y() - offset.y(), 
-                        end.x() - offset.x(), end.y() - offset.y());
+
+      bool useAsymmetricDouble = options & OBDepict::asymmetricDoubleBond;
+      if (HasLabel(beginAtom) && HasLabel(endAtom))
+        useAsymmetricDouble = false;
+      if (HasLabel(beginAtom) && endAtom->GetValence() == 3)
+        useAsymmetricDouble = false;
+      if (HasLabel(endAtom) && beginAtom->GetValence() == 3)
+        useAsymmetricDouble = false;
+
+
+      if (!useAsymmetricDouble) {
+        // style1
+        //
+        // -----------
+        // -----------
+        vector3 offset = orthogonalLine * 0.5 * bondSpacing;
+        painter->DrawLine(begin.x() + offset.x(), begin.y() + offset.y(),
+                          end.x() + offset.x(), end.y() + offset.y());
+        painter->DrawLine(begin.x() - offset.x(), begin.y() - offset.y(),
+                          end.x() - offset.x(), end.y() - offset.y());
+      } else {
+        // style2
+        //
+        //   -------
+        // -----------
+        vector3 offset1 = orthogonalLine * /*0.5 * */ bondSpacing;
+        vector3 offset2 = vb * /*0.5 * */ bondSpacing;
+        vector3 offset3 = - vb * /*0.5 * */ bondSpacing;
+
+        if (HasLabel(beginAtom))
+          offset2 = VZero;
+        if (HasLabel(endAtom))
+          offset3 = VZero;
+
+        painter->DrawLine(begin.x(), begin.y(), end.x(), end.y());
+        painter->DrawLine(begin.x() + offset1.x() + offset2.x(),
+                          begin.y() + offset1.y() + offset2.y(),
+                          end.x() + offset1.x() + offset3.x(),
+                          end.y() + offset1.y() + offset3.y());
+      }
     } else if (order == 3) {
       vector3 orthogonalLine = cross(end - begin, VZ).normalize();
       vector3 offset = orthogonalLine * 0.7 * bondSpacing;
@@ -579,9 +680,9 @@ namespace OpenBabel
 
     vector3 vbb = end - begin;
     if (HasLabel(beginAtom))
-      begin += 0.26 * vbb;
+      begin += 0.33 * vbb;
     if (HasLabel(endAtom))
-      end -= 0.26 * vbb;
+      end -= 0.33 * vbb;
     painter->DrawLine(begin.x(), begin.y(), end.x(), end.y());
 
     if (HasLabel(beginAtom))
@@ -594,9 +695,7 @@ namespace OpenBabel
 
   void OBDepictPrivate::DrawAtomLabel(const std::string &label, int alignment, const vector3 &pos)
   {
-    painter->SetFontSize(fontSize);
-    OBFontMetrics metrics = painter->GetFontMetrics(label);
-    /*
+   /*
     cout << "FontMetrics(" << label << "):" << endl;
     cout << "  ascent = " << metrics.ascent << endl;
     cout << "  descent = " << metrics.descent << endl;
@@ -609,40 +708,87 @@ namespace OpenBabel
     painter->SetPenColor(OBColor("black"));
     */
  
+    // compute the total width
+    double totalWidth = 0.0;
+    if ((alignment == Right) || (alignment == Left) || (label.find("H") == std::string::npos)) {
+      for (int i = 0; i < label.size(); ++i) {
+        if (!isalpha(label[i])) {
+          painter->SetFontSize(subscriptSize);
+          totalWidth += painter->GetFontMetrics(label.substr(i, 1)).width;
+        } else {
+          painter->SetFontSize(fontSize);
+          totalWidth += painter->GetFontMetrics(label.substr(i, 1)).width;
+        }
+      }
+    } else {
+      painter->SetFontSize(fontSize);
+      totalWidth = painter->GetFontMetrics(label.substr(0, label.find("H"))).width;
+      double width = 0.0; 
+      for (int i = label.find("H"); i < label.size(); ++i) {
+        if (!isalpha(label[i])) {
+          painter->SetFontSize(subscriptSize);
+          width += painter->GetFontMetrics(label.substr(i, 1)).width;
+        } else {
+          painter->SetFontSize(fontSize);
+          width += painter->GetFontMetrics(label.substr(i, 1)).width;
+        }
+      }
+
+      if (width > totalWidth)
+        totalWidth = width; 
+    }
+
+    painter->SetFontSize(fontSize);
+    OBFontMetrics metrics = painter->GetFontMetrics(label);
+ 
+
     std::string str, subscript;
     // compute the horizontal starting position
     double xOffset, yOffset, yOffsetSubscript;
     switch (alignment) {
-      case TopLeft:
-      case CenterLeft:
-      case BottomLeft:
-        xOffset = - 0.5 * painter->GetFontMetrics(label.substr(0,1)).width;
-        break;
-      case TopRight:
-      case CenterRight:
-      case BottomRight:
-        xOffset = 0.7 * painter->GetFontMetrics(label.substr(label.size()-1,1)).width - 
+      case Right:
+        xOffset = 0.5 * painter->GetFontMetrics(label.substr(0, 1)).width - 
                   painter->GetFontMetrics(label).width;
         break;
+      case Left:
+        xOffset = - 0.5 * painter->GetFontMetrics(label.substr(label.size()-1, 1)).width;
+        break;
+      case Up:
+      case Down:
+        if (label.find("H") != std::string::npos)
+          xOffset = - 0.5 * painter->GetFontMetrics(label.substr(0, label.find("H"))).width;
+        else
+          xOffset = - 0.5 * totalWidth;
       default:
-        xOffset = - 0.5 * painter->GetFontMetrics(label).width;
+        xOffset = - 0.5 * totalWidth;
         break;
     }
+
     // compute the vertical starting position
-    switch (alignment) {
-      case BottomLeft:
-      case BottomCenter:
-      case BottomRight:
-        yOffset = 0.2 * metrics.ascent + metrics.descent;
-        yOffsetSubscript = metrics.ascent + metrics.descent;
-        break;
-      default:
-        yOffset = 0.5 * metrics.ascent;
-        yOffsetSubscript = metrics.ascent;
-        break;
-    }
-    
+    yOffset = 0.5 * (metrics.ascent /*- metrics.descent*/);
+    yOffsetSubscript = yOffset - metrics.descent;
+    double xInitial = xOffset;
+
     for (int i = 0; i < label.size(); ++i) {
+      if (label[i] == 'H') {
+        if ((alignment == Up) || (alignment == Down))
+          if (!str.empty()) {
+            // write the current string
+            painter->SetFontSize(fontSize);
+            painter->DrawText(pos.x() + xOffset, pos.y() + yOffset, str);
+            if (alignment == Down) {
+              yOffset += metrics.fontSize;
+              yOffsetSubscript += metrics.fontSize;
+            } else {
+              yOffset -= metrics.fontSize;
+              yOffsetSubscript -= metrics.fontSize;
+            }
+            xOffset = xInitial;
+            str.clear();
+          }
+      }
+
+
       if (!isalpha(label[i])) {
         if (!str.empty()) {
           // write the current string
@@ -691,7 +837,14 @@ namespace OpenBabel
     return false;
   }
 
-  void OBDepictPrivate::SetWedgeAndHash(OBMol* mol)  {
+  void OBDepictPrivate::SetWedgeAndHash(OBMol* mol)  
+  {
+    // Remove any existing wedge and hash bonds
+    FOR_BONDS_OF_MOL(b,mol)  {
+      b->UnsetWedge();
+      b->UnsetHash();
+    }
+
     std::map<OBBond*, enum OBStereo::BondDirection> updown;
     std::map<OBBond*, OBStereo::Ref> from;
     std::map<OBBond*, OBStereo::Ref>::const_iterator from_cit;
