@@ -1,15 +1,15 @@
 /**********************************************************************
 descriptor.cpp - Implementation of the base class for molecular descriptors
- 
+
 Copyright (C) 2007 by Chris Morley
- 
+
 This file is part of the Open Babel project.
-For more information, see <http://openbabel.sourceforge.net/>
- 
+For more information, see <http://openbabel.org/>
+
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation version 2 of the License.
- 
+
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -24,25 +24,25 @@ GNU General Public License for more details.
 
 using namespace std;
 namespace OpenBabel
-{ 
+{
 #if defined(__CYGWIN__) || defined(__MINGW32__)
   // macro to implement static OBPlugin::PluginMapType& Map()
   PLUGIN_CPP_FILE(OBDescriptor)
 #endif
 
-/** 
+/**
      Compare() is a virtual function and can be overridden to allow different
-     comparison behaviour. 
+     comparison behaviour.
      The default implementation here is suitable for OBDescriptor classes
      which return a double value. The stringstream is parsed to retrieve a
      comparison operator, one of     > < >= <= = == != , and a numerical value.
      The function compares this the value returned by Predict() and returns
-     the result. The stringstream is left after the number, and its state 
+     the result. The stringstream is left after the number, and its state
      reflects whether any errors have occurred.
      If noEval is true, the parsing is as normal but Predict is not called
      and the function returns false.
  **/
-bool OBDescriptor::Compare(OBBase* pOb, istream& optionText, bool noEval)
+bool OBDescriptor::Compare(OBBase* pOb, istream& optionText, bool noEval, string* param)
 {
   //Get comparison operator
   char ch1=0, ch2=0;
@@ -50,7 +50,7 @@ bool OBDescriptor::Compare(OBBase* pOb, istream& optionText, bool noEval)
     optionText >> ch1;
   if(ispunctU(optionText.peek()))
     optionText >> ch2;
-  
+
   //Get number
   double filterval, val;
   optionText >> filterval;
@@ -58,17 +58,17 @@ bool OBDescriptor::Compare(OBBase* pOb, istream& optionText, bool noEval)
   {
     if(noEval)
       return false;
-    
-    val = Predict(pOb);
+
+    val = Predict(pOb, param);
 
     return DoComparison(ch1, ch2, val, filterval);
   }
   optionText.setstate(std::ios::badbit); //shows error
-  obErrorLog.ThrowError(__FUNCTION__, "Error in filter string" , obError);
+  obErrorLog.ThrowError(__FUNCTION__, "Error in filter string" , obError, onceOnly);
   return false;
 }
 
-/// Interprets the --filter option string and returns the combined result of all the comparisons it contains  
+/// Interprets the --filter option string and returns the combined result of all the comparisons it contains
 /**
     The string has the form:
     PropertyID1 predicate1 [booleanOp] PropertyID2 predicate2 ...
@@ -77,10 +77,10 @@ bool OBDescriptor::Compare(OBBase* pOb, istream& optionText, bool noEval)
     The predicates must start with a punctuation character and are interpreted by
     the Compare function of the OBDescriptor class. The default implementation expects
     a comparison operator and a number, e.g. >=1.3  Whitespace is optional and is ignored.
-    Each predicate and this OBBase object (usually OBMol) is passed to 
-    the Compare function of a OBDescriptor. The result of each comparison 
+    Each predicate and this OBBase object (usually OBMol) is passed to
+    the Compare function of a OBDescriptor. The result of each comparison
     is combined in a boolean expression (which can include parentheses)
-    in the normal way. The AND operator can be & or &&, the OR operator can be 
+    in the normal way. The AND operator can be & or &&, the OR operator can be
     | or ||, and a unitary NOT is !  The expected operator precedence
     is achieved using recursive calls of the function. If there is no boolean Op, all
     the tests have to return true for the function to return true, i.e. the default is AND.
@@ -110,7 +110,7 @@ bool OBDescriptor::FilterCompare(OBBase* pOb, std::istream& optionText, bool noE
       optionText >> ch;
       if(ch!=')')
       {
-        obErrorLog.ThrowError(__FUNCTION__, "Missing ')' in filter string", obError);
+        obErrorLog.ThrowError(__FUNCTION__, "Missing ')' in filter string", obError, onceOnly);
         return retFromCompare;
       }
     }
@@ -121,20 +121,22 @@ bool OBDescriptor::FilterCompare(OBBase* pOb, std::istream& optionText, bool noE
       else
       {
         string mes("Filter string has erroneous character : ");
-        obErrorLog.ThrowError(__FUNCTION__, mes + ch, obError);
+        obErrorLog.ThrowError(__FUNCTION__, mes + ch, obError, onceOnly);
         optionText.setstate(std::ios::badbit); //shows error
         return false;
       }
 
-      string descID = GetIdentifier(optionText);
+      pair<string,string> spair = GetIdentifier(optionText);
+      string descID = spair.first;
+      string param  = spair.second;
       if(descID.empty())
       {
         optionText.setstate(std::ios::badbit); //shows error
         return false;
       }
-      
+
       //If there is existing OBPairData use that
-      if(MatchPairData(pOb, descID))
+      if(param.empty() && MatchPairData(pOb, descID))
       {
         string value = pOb->GetData(descID)->GetValue();
         retFromCompare = CompareStringWithFilter(optionText, value, noEval, true);
@@ -144,7 +146,7 @@ bool OBDescriptor::FilterCompare(OBBase* pOb, std::istream& optionText, bool noE
         //if no existing data see if it is an OBDescriptor
         OBDescriptor* pDesc = OBDescriptor::FindType(descID.c_str());
         if(pDesc && !noEval)
-          retFromCompare = pDesc->Compare(pOb, optionText, noEval);
+          retFromCompare = pDesc->Compare(pOb, optionText, noEval, &param);
         else
         {
           //just parse
@@ -179,7 +181,7 @@ bool OBDescriptor::FilterCompare(OBBase* pOb, std::istream& optionText, bool noE
     else
       if(optionText.peek()==ch) //treat && and || as & and |
         optionText.ignore();
-        
+
     if(ch=='|')
     {
       retFromCompare = FilterCompare(pOb, optionText, ret || noEval);
@@ -192,27 +194,48 @@ bool OBDescriptor::FilterCompare(OBBase* pOb, std::istream& optionText, bool noE
 }
 
 //////////////////////////////////////////////////////////////
-string OBDescriptor::GetIdentifier(istream& optionText)
+pair<string,string> OBDescriptor::GetIdentifier(istream& optionText)
 {
-  string descID;
+  string descID, param;
   descID.clear();
   char ch;
   optionText >> ch; //ignore leading white space
   optionText.unsetf(ios::skipws);
   for(;;)
   {
-    if(!optionText || isspace(ch))
+    if(!optionText || isspace(ch) || ch==',')
       break;
-    if(ispunctU(ch))
+    if(ch=='(') // the parameter is in parentheses
+    {
+      ch = optionText.peek();
+      if(ch=='\"' || ch=='\'')
+      {
+        //parameter is in quotes
+        optionText.ignore(); // skip " or '
+        getline(optionText, param, ch);
+        optionText.ignore(numeric_limits<streamsize>::max(),')');
+      }
+      else
+        getline(optionText, param, ')');
+
+      if(!optionText)
+      {
+        obErrorLog.ThrowError(__FUNCTION__, "Missing ')' in descriptor parameter", obError, onceOnly);
+        descID.clear();
+        return make_pair(descID, descID);//both empty
+      }
+    }
+    else if(ispunctU(ch))
     {
       optionText.unget(); //put back char after ID
       break;
     }
-    descID.push_back(ch);
+    else
+      descID.push_back(ch);
     optionText >> ch;
   }
   optionText.setf(ios::skipws);
-  return descID;
+  return make_pair(descID, param);
 }
 
 
@@ -242,6 +265,8 @@ double OBDescriptor::ParsePredicate(istream& optionText, char& ch1, char& ch2, s
   //Try to read a double. Rewind and read as a string
   streampos spos = optionText.tellg();
   optionText >> val;
+  if(!optionText.eof()) //only a number when the param has no additional text
+    val = std::numeric_limits<double>::quiet_NaN();
   optionText.clear();
   optionText.seekg(spos);
   ReadStringFromFilter(optionText, svalue);
@@ -257,14 +282,13 @@ double OBDescriptor::ParsePredicate(istream& optionText, char& ch1, char& ch2, s
     mystring  =mystring ==mystring [must be terminated by a space or tab]
     "mystring" 'mystring'  ="mystring" ='mystring' [mystring can contain spaces or tabs]
     !=mystring !="mystring" [Returns false indicating negate]
-    There can be spaces or tabs after the operator = == !=  
+    There can be spaces or tabs after the operator = == !=
  **/
 bool OBDescriptor::ReadStringFromFilter(istream& optionText, string& result)
 {
-  bool error=false;
   bool ret=true;
   char ch;
-  
+
   if(optionText >> ch)
   {
     if(ch=='=' || ch=='!')
@@ -276,7 +300,7 @@ bool OBDescriptor::ReadStringFromFilter(istream& optionText, string& result)
     }
     else  //no operator
       optionText.unget();
-  
+
     optionText >> ch;
     if(ch=='\"' || ch=='\'')
     {
@@ -285,7 +309,6 @@ bool OBDescriptor::ReadStringFromFilter(istream& optionText, string& result)
     else // not quoted; get string up to next space or ')'
     {
       optionText.unget();
-      char ch;
       result.clear();
       optionText >> ch; //ignore leading white space
       optionText.unsetf(ios::skipws);
@@ -303,18 +326,18 @@ bool OBDescriptor::ReadStringFromFilter(istream& optionText, string& result)
       optionText.setf(ios::skipws);
     }
   }
-  
+
   if(optionText.fail())
-    obErrorLog.ThrowError(__FUNCTION__, "Error reading string from filter", obError);
-  
+    obErrorLog.ThrowError(__FUNCTION__, "Error reading string from filter", obError, onceOnly);
+
   return ret;
 }
 
-double OBDescriptor::PredictAndSave(OBBase* pOb)
+double OBDescriptor::PredictAndSave(OBBase* pOb, string* param)
 {
   string attr = GetID();
   string svalue;
-  double val = GetStringValue(pOb, svalue);
+  double val = GetStringValue(pOb,svalue, param );
 
   OBPairData *dp = static_cast<OBPairData *> (pOb->GetData(attr));
   bool PreviouslySet = true;
@@ -332,19 +355,16 @@ double OBDescriptor::PredictAndSave(OBBase* pOb)
 }
 
 /// This default version provides a string representation of the numeric value
-double OBDescriptor::GetStringValue(OBBase* pOb, string& svalue)
+double OBDescriptor::GetStringValue(OBBase* pOb, string& svalue, string* param)
 {
-  double val = Predict(pOb);
-  if(val!=std::numeric_limits<double>::quiet_NaN())
-  {
-    stringstream ss;
-    ss << val;
-    svalue = ss.str();
-  }
+  double val = Predict(pOb, param);
+  stringstream ss;
+  ss << val;
+  svalue = ss.str();
   return val;
 }
 
-bool OBDescriptor::CompareStringWithFilter(istream& optionText, string& sval, bool noEval, bool NoCompOK)
+bool OBDescriptor::CompareStringWithFilter(istream& optionText, string& sval, bool, bool NoCompOK)
 {
   char ch1=0, ch2=0;
   string sfilterval;
@@ -354,7 +374,7 @@ bool OBDescriptor::CompareStringWithFilter(istream& optionText, string& sval, bo
     // there is no comparison operator
     return true; // means that the identifier exists
   }
-  
+
   stringstream ss(sval);
   double val;
   if((ss >> val) && !IsNan(filterval))
@@ -386,7 +406,7 @@ bool OBDescriptor::CompareStringWithFilter(istream& optionText, string& sval, bo
       else
       {
         // '*' is some other character. Not not currently supported.
-        obErrorLog.ThrowError("--filter option", "Wild card * can only be the first or last character.", obError);
+        obErrorLog.ThrowError("--filter option", "Wild card * can only be the first or last character.", obError, onceOnly);
         return false;
       }
     }
@@ -396,16 +416,15 @@ bool OBDescriptor::CompareStringWithFilter(istream& optionText, string& sval, bo
 
 void OBDescriptor::AddProperties(OBBase* pOb, const string& DescrList)
 {
-  vector<string> vs;
-  tokenize(vs, DescrList.c_str(), " \t\r\n,/-*&;:|%+");
-  vector<string>::iterator itr;
-  for(itr=vs.begin();itr!=vs.end();++itr)
+  stringstream ss(DescrList);
+  OBDescriptor* pDescr;
+  while(ss)
   {
-    OBDescriptor* pDescr = OBDescriptor::FindType(itr->c_str());
-    if(pDescr)
-      pDescr->PredictAndSave(pOb);
+    pair<string,string> spair = GetIdentifier(ss);
+    if( (pDescr = OBDescriptor::FindType(spair.first.c_str())) ) // extra parentheses to indicate assignment as truth value
+      pDescr->PredictAndSave(pOb, &spair.second);
     else
-      obErrorLog.ThrowError(__FUNCTION__, *itr + " not recognized as a descriptor", obError);
+      obErrorLog.ThrowError(__FUNCTION__, spair.first + " not recognized as a descriptor", obError, onceOnly);
   }
 }
 
@@ -422,35 +441,38 @@ void OBDescriptor::DeleteProperties(OBBase* pOb, const string& DescrList)
 }
 
   //Reads list of descriptor IDs and OBPairData names and returns a list of values
-  //each precede by a space or the first character in the list if it is whitespace or punctuation.
+  //each preceded by a space or the first character in the list if it is whitespace or punctuation.
   //Used in OBMol::Transform() to append to title , but that is not done here to avoid
   //having to #include mol.h in this file.
   string OBDescriptor::GetValues(OBBase* pOb, const std::string& DescrList)
   {
-    vector<string> vs;
+    stringstream ss(DescrList);
     char delim = DescrList[0];
-    delim = isspace(delim)||ispunctU(delim) ? delim : ' ';
+    if(isspace(delim)||ispunctU(delim))
+      ss.ignore();//skip delim char
+    else
+      delim = ' ';
 
-    tokenize(vs, DescrList.c_str(), " \t\r\n,/-*&;:|%+");
-    vector<string>::iterator itr;
     string values;
-    for(itr=vs.begin();itr!=vs.end();++itr)
+    OBDescriptor* pDescr;
+    while(ss)
     {
       string thisvalue;
+      pair<string,string> spair = GetIdentifier(ss);
+
       //If there is existing OBPairData use that
-      if(MatchPairData(pOb,*itr))
-        thisvalue = pOb->GetData(*itr)->GetValue();
+      if(MatchPairData(pOb, spair.first))
+        thisvalue = pOb->GetData(spair.first)->GetValue();
       else
       {
-        //if it is an OBDescriptor
-        OBDescriptor* pDesc = OBDescriptor::FindType(itr->c_str());
-        if(!pDesc) 
+        if( (pDescr = OBDescriptor::FindType(spair.first.c_str())) ) // extra parentheses to indicate truth value
+          pDescr->GetStringValue(pOb, thisvalue, &spair.second);
+        else
         {
-          obErrorLog.ThrowError(__FUNCTION__, 
-            *itr + " not recognized as either a property or a descriptor", obError);
-          return values;
+          obErrorLog.ThrowError(__FUNCTION__,
+            spair.first + " not recognized as a property or a descriptor", obError, onceOnly);
+          thisvalue = "??";
         }
-        pDesc->GetStringValue(pOb, thisvalue);
       }
       values += delim + thisvalue;
     }
@@ -467,7 +489,7 @@ void OBDescriptor::DeleteProperties(OBBase* pOb, const string& DescrList)
     if(s.find('_')==string::npos)
       return false;
     string temp(s);
-    string::size_type pos=-1;
+    string::size_type pos = string::npos;
     //replace all underscores by spaces
     while((pos=temp.find('_', ++pos))!=string::npos)
       temp[pos]=' ';
@@ -500,7 +522,7 @@ bool OBDescriptor::Display(std::string&txt, const char* param, const char* ID)
   \since version 2.2
 
 OBDescriptor and Filtering
-           
+
 On the command line, using the option --filter filter-string converts only
 those molecules which meet the criteria specified in the filter-string. This
 is useful to select particular molecules from a set.
@@ -527,7 +549,7 @@ babel -L descriptors
 or from the functions OBPlugin::List, OBPlugin::ListAsString and OBPlugin::ListAsVector.
 
 The filter-string is interpreted by a static function of OBDescriptor,
-FilterCompare(). This identifies the descriptor IDs and then calls a virtual 
+FilterCompare(). This identifies the descriptor IDs and then calls a virtual
 function, Compare(), of each OBDescriptor class to interpret the rest of relational
 expression, for example, ">200", or "=c1ccccc1". The default version of Compare()
 is suitable for descriptors, like MW or logP, which return a double from
@@ -537,7 +559,7 @@ provide their own.
 By default, as in the example, OBDescriptor::FilterCompare() would AND each
 comparison so that all the comparisons must be true for the test to succeed.
 However filter-string could also be a full boolean expression, with &, |, !,
-and parenthases allowing any combination of features to be selected. 
+and parenthases allowing any combination of features to be selected.
 FilterCompareAs calls itself recursively to give AND precidence over OR and
 evaluation is not carried out if not needed.
 
@@ -548,31 +570,40 @@ that are reasonable.
 The base class, OBDescriptor, uses pointers to OBBase in its functions,
 like OBFormat, to improve extendability - reactions could have
 features too. It does mean that a dynamic_cast is needed at the start of the
-Predict(OBBase* pOb) functions.
+Predict(OBBase* pOb, string*) functions.
 
 To use a particular descriptor, like logP, when programming with the API, use
 code like the following:
 \code
   OBDescriptor* pDescr = OBDecriptor::FindType("logP");
   if(pDescr)
-    double val = pDescr->Predict(mol);
+    double val = pDescr->Predict(mol, param);
 \endcode
 To add the descriptor ID and the predicted data to OBPairData attached to
 the object, use PredictAndSave().
 
+Descriptors can have a string parameter, which each descriptor can interpret
+as it wants, maybe, for instance as multiple numeric values. The parameter
+is in brackets after the descriptor name, e.g. popcount(FP4). In the above
+programming example param is a pointer to a std::string which has a default
+value of NULL, meaning no parameter. GetStringValue() and Compare() are similar.
+
+To parse a string for descriptors use GetIdentifier(), which returns both
+the ID and the parameter, if there is one.
+
 This facility can be called from the command line.Use the option
---add "descriptor list", which will add the requested descriptors to the 
-molecule.  They are then visible as properties in SDF and CML formats. 
-The IDs in the list can be separated by spaces or punctuation characters.
+--add "descriptor list", which will add the requested descriptors to the
+molecule.  They are then visible as properties in SDF and CML formats.
+The IDs in the list can be separated by spaces or commas.
 All Descriptors will provide an output value as a string through a  virtual
 function GetStringValue((OBBase* pOb, string& svalue)) which
 assigns the value of a string descriptor(like inchi) to svalue or a string
 representation of a numerical property like logP.
 
-The classes MWFilter and TitleFilter illustrate the code that has to be 
+The classes MWFilter and TitleFilter illustrate the code that has to be
 provided for numerical and non-numerical descriptors.
 
-*/ 
+*/
 
 }//namespace
 
