@@ -39,11 +39,18 @@ public:
   {
     return
     "MCDL format\n"
-    "Modular Chemical Descriptor Language\n"
-    "As described in Gakh A.A., Burnett M.N.,\n"
-    "Modular Chemical Descriptor Language (MCDL):\n"
-    "Composition, Connectivity and Supplementary Modules\n"
-    "J.Chem.Inf.Comput.Sci, 2001, 41, 1494-1499";
+    "Modular Chemical Descriptor Language\n\n"
+    "As described in [gb2001]_.\n\n"
+
+".. [gb2001] A.A. Gakh and M.N. Burnett. **Modular Chemical Descriptor\n"
+"            Language (MCDL): Composition, Connectivity and\n"
+"            Supplementary Modules.**\n"
+"            *J. Chem. Inf. Comput. Sci.*, **2004**, *41*, 1491-1499.\n"
+"            [`Link <http://dx.doi.org/10.1021/ci000108y>`_]\n\n"
+
+"Here's an example conversion from SMILES to MCDL::\n\n"
+"  obabel -:\"CC(=O)Cl\" -omcdl\n"
+"  CHHH;COCl[2]\n";
   }
 
   virtual const char* SpecificationURL(){return
