@@ -52,12 +52,21 @@ namespace OpenBabel
     {
       return
         "Crystallographic Information File\n"
-        "The CIF file format is the standard interchange format for small-molecule chemical structures\n\n"
+        "The CIF file format is the standard interchange format for small-molecule crystal structures\n\n"
+        "Fractional coordinates are converted to cartesian ones using the following convention:\n\n"
+
+        "- The x axis is parallel to a\n"
+        "- The y axis is in the (a,b) plane\n"
+        "- The z axis is along c*\n\n"
+
+        "Ref: Int. Tables for Crystallography (2006), vol. B, sec 3.3.1.1.1\n"
+        "  (the matrix used is the 2nd form listed)\n\n"
+
         "Read Options e.g. -ab:\n"
         "  v  Verbose CIF conversion\n"
         "  s  Output single bonds only\n"
         "  b  Disable bonding entirely\n"
-        "  B  Use bonds listed in CIF file from _geom_bond_* records (overrides option b) \n\n";
+        "  B  Use bonds listed in CIF file from _geom_bond_etc records (overrides option b) \n\n";
     };
 
     virtual const char* SpecificationURL()
@@ -239,6 +248,8 @@ namespace OpenBabel
     /// Cartesian2Fractionnal matrix
     float mOrthMatrixInvert[3][3];
     const SpaceGroup *mSpaceGroup;
+    /// Name for the CIF data block
+    std::string mDataBlockName;
   };
   /** Main CIF class - parses the stream and separates data blocks, comments, items, loops.
    * All values are stored as string, and Each CIF block is stored in a separate CIFData object.
@@ -277,22 +288,67 @@ namespace OpenBabel
 
   void CIFData::ExtractAll(const bool verbose)
   {
+    {
+      stringstream ss;
+      ss<<"CIF: interpreting data block: "<<mDataBlockName;
+      obErrorLog.ThrowError(__FUNCTION__, ss.str(), obInfo);
+    }
+    if(mDataBlockName=="data_global")
+      { // :KLUDGE: this data block name is used for journal &author information
+        // for IUCr journals, so do not generate an error if the block contains
+        // no structural information
+        bool empty_iucrjournal_block=true;
+        if(mvItem.find("_cell_length_a"   )!=mvItem.end()) empty_iucrjournal_block=false;
+        if(mvItem.find("_cell_length_b"   )!=mvItem.end()) empty_iucrjournal_block=false;
+        if(mvItem.find("_cell_length_c"   )!=mvItem.end()) empty_iucrjournal_block=false;
+        for(map<set<ci_string>,map<ci_string,vector<string> > >::const_iterator loop=mvLoop.begin();
+            loop!=mvLoop.end();++loop)
+          {
+            if(loop->second.find("_atom_site_fract_x")!=loop->second.end()) empty_iucrjournal_block=false;
+            if(loop->second.find("_atom_site_fract_y")!=loop->second.end()) empty_iucrjournal_block=false;
+            if(loop->second.find("_atom_site_fract_z")!=loop->second.end()) empty_iucrjournal_block=false;
+            if(loop->second.find("_atom_site_Cartn_x")!=loop->second.end()) empty_iucrjournal_block=false;
+            if(loop->second.find("_atom_site_Cartn_y")!=loop->second.end()) empty_iucrjournal_block=false;
+            if(loop->second.find("_atom_site_Cartn_z")!=loop->second.end()) empty_iucrjournal_block=false;
+          }
+        if(empty_iucrjournal_block) 
+          {
+            stringstream ss;
+            ss << "CIF WARNING: found en empty 'data_global' block - SKIPPING\n"
+               << "  (you can safely ignore this if reading a CIF file from an IUCr journal)";
+            obErrorLog.ThrowError(__FUNCTION__, ss.str(), obWarning);
+            return;
+          }
+    }
     // :@todo: Take care of values listed as "." and "?" instead of a real value.
     this->ExtractName(verbose);
-    this->ExtractUnitCell(verbose);
+    // Spacegroup must be extracted before unit cell
     this->ExtractSpacegroup(verbose);
+    this->ExtractUnitCell(verbose);
     this->ExtractAtomicPositions(verbose);
+    if(mvAtom.size()==0)
+      {
+        stringstream ss;
+        ss << "CIF Error: no atom found ! (in data block:"<<mDataBlockName<<")";
+        obErrorLog.ThrowError(__FUNCTION__, ss.str(), obError);
+      }
     this->ExtractBonds(verbose);
   }
 
   void CIFData::ExtractUnitCell(const bool verbose)
   {
-    map<ci_string,string>::const_iterator positem;
-    positem=mvItem.find("_cell_length_a");
-    if(positem!=mvItem.end())
+    // Use spacegroup to determine missing angle or length
+    const int spgid= mSpaceGroup->GetId();
+    if(  (mvItem.find("_cell_length_a")!=mvItem.end())
+       ||(mvItem.find("_cell_length_b")!=mvItem.end())
+       ||(mvItem.find("_cell_length_c")!=mvItem.end()) )
       {
         mvLatticePar.resize(6);
-        mvLatticePar[0]=CIFNumeric2Float(positem->second);
+        for(unsigned int i=0;i<6;i++) mvLatticePar[i]=float(0);
+        map<ci_string,string>::const_iterator positem;
+        positem=mvItem.find("_cell_length_a");
+        if(positem!=mvItem.end())
+          mvLatticePar[0]=CIFNumeric2Float(positem->second);
         positem=mvItem.find("_cell_length_b");
         if(positem!=mvItem.end())
           mvLatticePar[1]=CIFNumeric2Float(positem->second);
@@ -311,10 +367,111 @@ namespace OpenBabel
         if(verbose) cout<<"Found Lattice parameters:" <<mvLatticePar[0]<<" , "<<mvLatticePar[1]<<" , "<<mvLatticePar[2]
                         <<" , "<<mvLatticePar[3]<<" , "<<mvLatticePar[4]<<" , "<<mvLatticePar[5]<<endl;
         mvLatticePar[3] = static_cast<float> (mvLatticePar[3] * DEG_TO_RAD);// pi/180
-		mvLatticePar[4] = static_cast<float> (mvLatticePar[4] * DEG_TO_RAD);
-		mvLatticePar[5] = static_cast<float> (mvLatticePar[5] * DEG_TO_RAD);
+        mvLatticePar[4] = static_cast<float> (mvLatticePar[4] * DEG_TO_RAD);
+        mvLatticePar[5] = static_cast<float> (mvLatticePar[5] * DEG_TO_RAD);
+        
+        // Fill values depending on spacegroup, *only* when missing
+        if((spgid>2)&&(spgid<=15))
+        {// :TODO: monoclinic spg, depending on unique axis....
+        }
+        if((spgid>15)&&(spgid<=142))
+        {// orthorombic & tetragonal
+          if(mvLatticePar[3]==0) mvLatticePar[3]=M_PI/2;
+          if(mvLatticePar[4]==0) mvLatticePar[4]=M_PI/2;
+          if(mvLatticePar[5]==0) mvLatticePar[5]=M_PI/2;
+        }
+        if((spgid>74)&&(spgid<=142))
+        {// Tetragonal, make sure a=b if one is missing
+          if(mvLatticePar[1]==0) mvLatticePar[1]=mvLatticePar[0];
+          if(mvLatticePar[0]==0) mvLatticePar[0]=mvLatticePar[1];
+        }
+        if((spgid>142)&&(spgid<=194))
+        {// trigonal/ rhomboedric...
+          const string::size_type pos = mSpaceGroup->GetHallName().find('R');
+          if(pos==std::string::npos)
+          {//rhomboedric cell, a=b=c, alpha=beta=gamma
+            float a=0;
+            if(mvLatticePar[0]>a) a=mvLatticePar[0];
+            if(mvLatticePar[1]>a) a=mvLatticePar[1];
+            if(mvLatticePar[2]>a) a=mvLatticePar[2];
+            if(mvLatticePar[0]==0) mvLatticePar[0]=a;
+            if(mvLatticePar[1]==0) mvLatticePar[1]=a;
+            if(mvLatticePar[2]==0) mvLatticePar[2]=a;
+            
+            float alpha=0;
+            if(mvLatticePar[3]>alpha) alpha=mvLatticePar[3];
+            if(mvLatticePar[4]>alpha) alpha=mvLatticePar[4];
+            if(mvLatticePar[5]>alpha) alpha=mvLatticePar[5];
+            if(mvLatticePar[3]==0) mvLatticePar[3]=alpha;
+            if(mvLatticePar[4]==0) mvLatticePar[4]=alpha;
+            if(mvLatticePar[5]==0) mvLatticePar[5]=alpha;
+          }
+          else
+          {//hexagonal cell, a=b & alpha=beta=pi/2, gamma= 2*pi/3
+            if(mvLatticePar[1]==0) mvLatticePar[1]=mvLatticePar[0];
+            if(mvLatticePar[0]==0) mvLatticePar[0]=mvLatticePar[1];
+            if(mvLatticePar[3]==0) mvLatticePar[3]=M_PI/2;
+            if(mvLatticePar[4]==0) mvLatticePar[4]=M_PI/2;
+            if(mvLatticePar[5]==0) mvLatticePar[5]=2*M_PI/3;
+          }
+        }
+        if(spgid>194)
+        {
+          if(mvLatticePar[3]==0) mvLatticePar[3]=M_PI/2;
+          if(mvLatticePar[4]==0) mvLatticePar[4]=M_PI/2;
+          if(mvLatticePar[5]==0) mvLatticePar[5]=M_PI/2;
+          // In case some idiot cif only supplies one value, make sure a=b=c
+          float a=0;
+          if(mvLatticePar[0]>a) a=mvLatticePar[0];
+          if(mvLatticePar[1]>a) a=mvLatticePar[1];
+          if(mvLatticePar[2]>a) a=mvLatticePar[2];
+          if(mvLatticePar[0]==0) mvLatticePar[0]=a;
+          if(mvLatticePar[1]==0) mvLatticePar[1]=a;
+          if(mvLatticePar[2]==0) mvLatticePar[2]=a;
+        }
+        // Handle missing values
+        if(mvLatticePar[3]<1e-6)
+        {
+            stringstream ss;
+            ss << "CIF WARNING: missing alpha value, defaulting to 90 degrees (in data block:"<<mDataBlockName<<")";
+            obErrorLog.ThrowError(__FUNCTION__, ss.str(), obWarning);
+            mvLatticePar[3]=90*DEG_TO_RAD;
+        }
+        if(mvLatticePar[4]<1e-6)
+        {
+            stringstream ss;
+            ss << "CIF WARNING: missing beta value, defaulting to 90 degrees (in data block:"<<mDataBlockName<<")";
+            obErrorLog.ThrowError(__FUNCTION__, ss.str(), obWarning);
+            mvLatticePar[4]=90*DEG_TO_RAD;
+        }
+        if(mvLatticePar[5]<1e-6)
+        {
+            stringstream ss;
+            ss << "CIF WARNING: missing gamma value, defaulting to 90 degrees (in data block:"<<mDataBlockName<<")";
+            obErrorLog.ThrowError(__FUNCTION__, ss.str(), obWarning);
+            mvLatticePar[5]=90*DEG_TO_RAD;
+        }
+        if(mvLatticePar[1]<1e-6)
+        {
+            stringstream ss;
+            ss << "CIF Error: missing b lattice parameter - cannot interpret structure ! (in data block:"<<mDataBlockName<<")";
+            obErrorLog.ThrowError(__FUNCTION__, ss.str(), obError);
+        }
+        if(mvLatticePar[2]<1e-6)
+        {
+            stringstream ss;
+            ss << "CIF Error: missing c lattice parameter - cannot interpret structure ! (in data block:"<<mDataBlockName<<")";
+            obErrorLog.ThrowError(__FUNCTION__, ss.str(), obError);
+        }
+
         this->CalcMatrices();
       }
+      else
+      {
+         stringstream ss;
+         ss << "CIF Error: missing a,b and c value - cannot interpret structure ! (in data block:"<<mDataBlockName<<")";
+         obErrorLog.ThrowError(__FUNCTION__, ss.str(), obError);
+      }
   }
 
   void CIFData::ExtractSpacegroup(const bool verbose)
@@ -386,16 +543,24 @@ namespace OpenBabel
           }
       }
     mSpaceGroup=NULL;
-    if (mSpacegroupNumberIT != 0) {
-      mSpaceGroup = SpaceGroup::GetSpaceGroup(mSpacegroupNumberIT);
-    }
-    else if (mSpacegroupSymbolHall.length() > 0) {
+    // be forgiving - if spg not found, try again
+    // Prefer Hall > HM == number, as Hall symbol is truly unique
+    if (mSpacegroupSymbolHall.length() > 0) {
+      //Make sure there are no leading spaces before Hall symbol (kludge)
+      for(std::string::iterator pos=mSpacegroupSymbolHall.begin();pos!=mSpacegroupSymbolHall.end();)
+      {
+        if((char)(*pos)==' ')  pos=mSpacegroupSymbolHall.erase(pos);
+        else pos++;
+      }
       mSpaceGroup = SpaceGroup::GetSpaceGroup(mSpacegroupSymbolHall);
     }
-    else if (mSpacegroupHermannMauguin.length() > 0) {
+    if((mSpaceGroup == NULL)&& (mSpacegroupHermannMauguin.length() > 0)) {
       mSpaceGroup = SpaceGroup::GetSpaceGroup(mSpacegroupHermannMauguin);
     }
-    else {
+    if((mSpaceGroup == NULL)&&(mSpacegroupNumberIT != 0)) {
+      mSpaceGroup = SpaceGroup::GetSpaceGroup(mSpacegroupNumberIT);
+    }
+    if(mSpaceGroup == NULL) {
       SpaceGroup *sg = new SpaceGroup();
       positem=mvItem.find("_symmetry_equiv_pos_as_xyz");
       if(positem!=mvItem.end())
@@ -424,9 +589,15 @@ namespace OpenBabel
         delete sg;
       }
     }
-    if (mSpaceGroup != NULL)
-      // set the space group name to Hall symbol
-      mSpacegroupSymbolHall = mSpaceGroup->GetHallName();
+    if(mSpaceGroup == NULL)
+    {
+        stringstream ss;
+        ss << "CIF Error: missing spacegroup description: defaulting to P1... (in data block:"<<mDataBlockName<<")";
+        obErrorLog.ThrowError(__FUNCTION__, ss.str(), obWarning);
+        mSpaceGroup = SpaceGroup::GetSpaceGroup(1);
+    }
+    // set the space group name to Hall symbol
+    mSpacegroupSymbolHall = mSpaceGroup->GetHallName();
   }
 
   void CIFData::ExtractName(const bool verbose)
@@ -875,6 +1046,7 @@ namespace OpenBabel
             block=tmp.substr(5);
             if(vv) cout<<endl<<endl<<"NEW BLOCK DATA: !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ->"<<block<<endl<<endl<<endl;
             mvData[block]=CIFData();
+            mvData[block].mDataBlockName=tmp;
             continue;
           }
         if((in.peek()=='l') || (in.peek()=='L'))
@@ -1096,6 +1268,7 @@ namespace OpenBabel
     for(map<string,CIFData>::iterator pos=cif.mvData.begin();pos!=cif.mvData.end();++pos)
       if(pos->second.mvAtom.size()>0)
         {
+          pmol->BeginModify();
           if(pos->second.mvLatticePar.size()==6)
             {// We have one unit cell
               string spg=pos->second.mSpacegroupSymbolHall;
