@@ -302,7 +302,7 @@ namespace OpenBabel
     std::vector<double> Frequencies, Intensities;
     //Rotational data
     std::vector<double> RotConsts(3);
-    int RotSymNum;
+    int RotSymNum=1;
     OBRotationData::RType RotorType;
 
     // Translation vectors (if present)
@@ -316,10 +316,53 @@ namespace OpenBabel
     // Orbital data
     std::vector<double> orbitals;
     std::vector<std::string> symmetries;
-    int aHOMO, bHOMO;
+    int aHOMO, bHOMO, betaStart;
+
+    //Put some metadata into OBCommentData
+    string comment("Gaussian ");
+    ifs.getline(buffer,BUFF_SIZE);
+    if(*buffer)
+    {
+      comment += strchr(buffer,'=')+2;
+      comment += "";
+      for(unsigned i=0; i<115, ifs; ++i)
+      {
+        ifs.getline(buffer,BUFF_SIZE);
+        if(buffer[1]=='#')
+        {
+          //the line describing the method
+          comment += buffer;
+          OBCommentData *cd = new OBCommentData;
+          cd->SetData(comment);
+          cd->SetOrigin(fileformatInput);
+          mol.SetData(cd);
+          break;
+        }
+      }
+    }
+
+    int i=0;
+    char coords_type[25];
+
+    //Prescan file to find second instance of "orientation:"
+    //This will be the kind of coords used in the chk/fchk file
+    while (ifs.getline(buffer,BUFF_SIZE))
+      {
+        if (strstr(buffer,"orientation:") !=NULL)
+           i++;
+        if (i==2)
+           break;
+     }
+    if (!ifs.eof())
+    {
+    tokenize (vs, buffer);
+    strcpy (coords_type, vs[0].c_str());
+    strcat (coords_type, " orientation:");
+    ifs.seekg(0);  //rewind
+    //    cout << coords_type << std::endl;
+    }
 
     mol.BeginModify();
-    bool have_coords = 0;
     while (ifs.getline(buffer,BUFF_SIZE))
       {
         if (strstr(buffer,"Multiplicity") != NULL)
@@ -333,12 +376,8 @@ namespace OpenBabel
 
             ifs.getline(buffer,BUFF_SIZE);
           }
-        else if((strstr(buffer,"Input orientation:") != NULL)
-                || ((strstr(buffer,"Standard orientation:") != NULL) && (!have_coords))
-                || ((strstr(buffer,"Z-Matrix orientation:") != NULL) && (!have_coords)))
+        else if (strstr(buffer, coords_type) != NULL)
           {
-            if (strstr(buffer,"Input orientation:") != NULL)
-              have_coords = 1; // if we came here from "Input orientation", disable reading "Standard orientation"
             numTranslationVectors = 0; // ignore old translationVectors
             ifs.getline(buffer,BUFF_SIZE);      // ---------------
             ifs.getline(buffer,BUFF_SIZE);      // column headings
@@ -500,15 +539,16 @@ namespace OpenBabel
           ifs.getline(buffer,BUFF_SIZE); //symmetry number
           tokenize(vs, buffer);
           RotSymNum = atoi(vs[3].c_str());
-          do
-          {
-            ifs.getline(buffer,BUFF_SIZE);
-          }while(ifs && !strstr(buffer, "Rotational constant"));
-          tokenize(vs, buffer);
-          for(int i=3; i<vs.size(); ++i)
-            RotConsts[i-3] = atof(vs[i].c_str());
-
         }
+
+        else if(strstr(buffer, "Rotational constant"))
+        {
+          tokenize(vs, buffer);
+          RotConsts.clear();
+          for(int i=3; i<vs.size(); ++i)
+            RotConsts.push_back(atof(vs[i].c_str()));
+        }
+
         else if(strstr(buffer, "alpha electrons")) // # of electrons / orbital
         {
           tokenize(vs, buffer);
@@ -524,35 +564,42 @@ namespace OpenBabel
             std::string label; // used as a temporary to remove "(" and ")" from labels
             int offset = 0;
 
-            ifs.getline(buffer, BUFF_SIZE);
-            tokenize(vs, buffer); // parse first line "Occupied" ...
-            for (unsigned int i = 1; i < vs.size(); ++i) {
-              label = vs[i].substr(1, vs[i].length() - 2);
-              symmetries.push_back(label);
-            }
-            ifs.getline(buffer, BUFF_SIZE);
-
-            // Parse remaining lines
-            while (strstr(buffer, "(")) {
-              tokenize(vs, buffer);
-              if (strstr(buffer, "Virtual")) {
-                offset = 1; // skip first token
-              } else {
-                offset = 0;
-              }
-              for (unsigned int i = offset; i < vs.size(); ++i) {
+            while(true) {
+              ifs.getline(buffer, BUFF_SIZE);
+              tokenize(vs, buffer); // parse first line "Occupied" ...
+              for (unsigned int i = 1; i < vs.size(); ++i) {
                 label = vs[i].substr(1, vs[i].length() - 2);
                 symmetries.push_back(label);
               }
-              ifs.getline(buffer, BUFF_SIZE); // get next line
-            } // end parsing symmetry labels
+              ifs.getline(buffer, BUFF_SIZE);
+
+              // Parse remaining lines
+              while (strstr(buffer, "(")) {
+                tokenize(vs, buffer);
+                if (strstr(buffer, "Virtual")) {
+                  offset = 1; // skip first token
+                } else {
+                  offset = 0;
+                }
+                for (unsigned int i = offset; i < vs.size(); ++i) {
+                  label = vs[i].substr(1, vs[i].length() - 2);
+                  symmetries.push_back(label);
+                }
+                ifs.getline(buffer, BUFF_SIZE); // get next line
+              } // end parsing symmetry labels
+              if (!strstr(buffer, "Beta")) // no beta orbitals
+                break;
+            } // end alpha/beta section
           }
         else if (strstr(buffer, "Alpha") && strstr(buffer, ". eigenvalues --")) {
           orbitals.clear();
+          betaStart = 0;
           while (strstr(buffer, ". eigenvalues --")) {
             tokenize(vs, buffer);
             if (vs.size() < 4)
               break;
+            if (vs[0].find("Beta") !=string::npos && betaStart == 0) // mark where we switch from alpha to beta
+              betaStart = orbitals.size();
             for (unsigned int i = 4; i < vs.size(); ++i) {
               orbitals.push_back(atof(vs[i].c_str()));
             }
@@ -631,7 +678,7 @@ namespace OpenBabel
 
         else if(strstr(buffer,"SCF Done:") != NULL)
           {
-#define HARTREE_TO_KCAL 627.509
+#define HARTREE_TO_KCAL 627.509469
             tokenize(vs,buffer);
             mol.SetEnergy(atof(vs[4].c_str()) * HARTREE_TO_KCAL);
             confEnergies.push_back(mol.GetEnergy());
@@ -676,6 +723,23 @@ namespace OpenBabel
         OBOrbitalData *od = new OBOrbitalData;
         if (aHOMO == bHOMO) {
           od->LoadClosedShellOrbitals(orbitals, symmetries, aHOMO);
+        } else {
+          // we have to separate the alpha and beta vectors
+          std::vector<double>      betaOrbitals;
+          std::vector<std::string> betaSymmetries;
+          int initialSize = orbitals.size();
+          for (unsigned int i = betaStart; i < initialSize; ++i) {
+            betaOrbitals.push_back(orbitals[i]);
+            betaSymmetries.push_back(symmetries[i]);
+          }
+          // ok, now erase the end elements of orbitals and symmetries
+          for (unsigned int i = betaStart; i < initialSize; ++i) {
+            orbitals.pop_back();
+            symmetries.pop_back();
+          }
+          // and load the alphas and betas
+          od->LoadAlphaOrbitals(orbitals, symmetries, aHOMO);
+          od->LoadBetaOrbitals(betaOrbitals, betaSymmetries, bHOMO);
         }
         od->SetOrigin(fileformatInput);
         mol.SetData(od);
