@@ -1,13 +1,13 @@
 /**********************************************************************
 Copyright (C) 2004 by Chris Morley
- 
+
 This file is part of the Open Babel project.
-For more information, see <http://openbabel.sourceforge.net/>
- 
+For more information, see <http://openbabel.org/>
+
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation version 2 of the License.
- 
+
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -16,12 +16,13 @@ GNU General Public License for more details.
 #include "openbabel/babelconfig.h"
 #include <string>
 #include <iomanip>
+#include <typeinfo>
 #include "openbabel/mol.h"
 #include "openbabel/obconversion.h"
 #include "openbabel/reaction.h"
 
 using namespace std;
-using std::tr1::shared_ptr;
+//using std::tr1::shared_ptr;
 
 namespace OpenBabel
 {
@@ -37,11 +38,13 @@ public:
   virtual const char* Description()
   {
       return
-          "MDL RXN format\n \
-          \n";
+        "MDL RXN format\n"
+        "The MDL reaction format is used to store information on chemical reactions.\n"
+        "Output Options, e.g. -xA\n"
+        " A  output in Alias form, e.g. Ph, if present\n\n";
   };
 
-  virtual const char* GetMIMEType() 
+  virtual const char* GetMIMEType()
   { return "chemical/x-mdl-rxn"; };
 
   virtual const char* TargetClassDescription()
@@ -76,11 +79,15 @@ public:
               obAuditMsg);
 
     if(ret) //Do transformation and return molecule
-      return pConv->AddChemObject(pReact->DoTransformations(pConv->GetOptions(OBConversion::GENOPTIONS)))!=0;
+      return pConv->AddChemObject(pReact->DoTransformations(pConv->GetOptions(OBConversion::GENOPTIONS),pConv))!=0;
     else
-        pConv->AddChemObject(NULL);
-    return false;
-  };
+    {
+      pConv->AddChemObject(NULL);
+      delete pReact;
+      pReact=NULL;
+      return false;
+    }
+};
 
   virtual bool WriteChemObject(OBConversion* pConv)
   {
@@ -114,10 +121,10 @@ bool RXNFormat::ReadMolecule(OBBase* pOb, OBConversion* pConv)
 {
     //It's really a reaction, not a molecule.
     //Doesn't make a new OBReactionObject, but does make mew reactant and product OBMols
-    OBReaction* pReact = dynamic_cast<OBReaction*>(pOb);
+   OBReaction* pReact = pOb->CastAndClear<OBReaction>();
 
     OBFormat* pMolFormat = pConv->FindFormat("MOL");
-    if(pMolFormat==NULL)
+    if(pMolFormat==NULL || !pReact)
         return false;
 
     //	OBConversion MolConv(*pConv); //new copy to use to read associated MOL
@@ -196,7 +203,7 @@ bool RXNFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
     OBConversion MolConv(*pConv); //new copy to use to write associated MOL
     MolConv.AddOption("no$$$$",OBConversion::OUTOPTIONS);
     MolConv.SetAuxConv(NULL); //temporary until a proper OBConversion copy constructor written
-     
+
     OBFormat* pMolFormat = pConv->FindFormat("MOL");
     if(pMolFormat==NULL)
     {
