@@ -70,17 +70,15 @@ namespace OpenBabel
     {
       return
         "Chemical Markup Language\n"
-        "An XML format.\n"
+        "An XML format for interchange of chemical information.\n\n"
 
         "This format writes and reads CML XML files. To write CML1 format rather than\n"
-        "the default CML2, use the -x1 option. To write the array form use -xa and to\n"
-        "specify all hydrogens using the hydrogenCount attribute on atoms use -xh.\n\n"
+        "the default CML2, use the ``-x1`` option. To write the array form use ``-xa``\n"
+        "and to specify all hydrogens using the hydrogenCount attribute on atoms use\n"
+        "``-xh``.\n\n"
 
-        "Crystal structures are written using the <crystal>, <xfract>) etc., elements\n"
-        "if the OBMol has a OBGenericDataType::UnitCell data.\n\n"
-
-        "If the OBMol has no bonds, a <formula> element is written instead of the\n"
-        "normal <atomArray> and <atom> elements.\n\n"
+        "Crystal structures are written using the <crystal>, <xfract> (,...etc.)\n"
+        "elements if the OBMol has a OBGenericDataType::UnitCell data.\n\n"
 
         "All these forms are handled transparently during reading. Only a subset of\n"
         "CML elements and attributes are recognised, but these include most of those\n"
@@ -129,8 +127,8 @@ namespace OpenBabel
         "defining the object that OpenBabel will convert. For CML this is\n"
         "<molecule>. Files can have multiple objects and these can be treated\n"
         "the same as with other multiple object formats like SMILES and MDL\n"
-        "Molfile. So conversion can start at the nth object using the -fn option\n"
-        "and finish before the end using the -ln option. Multiple object XML files\n"
+        "Molfile. So conversion can start at the nth object using the ``-fn`` option\n"
+        "and finish before the end using the ``-ln`` option. Multiple object XML files\n"
         "also can be indexed and searched using FastSearch, although this has\n"
         "not yet been extensively tested.\n\n";
     };
@@ -165,7 +163,7 @@ namespace OpenBabel
 
     void MakeAtomIds(OBMol& mol, vector<string>& atomIDs);
     void WriteFormula(OBMol mol); //passes copy of mol
-    void WriteMetadataList();
+    void WriteMetadataList(OBMol& mol);
     string getTimestr();
     void WriteBondStereo(OBBond* pbond, vector<string>& atomIDs);
     void WriteCrystal(OBMol& mol);
@@ -368,7 +366,18 @@ namespace OpenBabel
 
             xmlTextReaderRead(reader());
             const xmlChar* pvalue = xmlTextReaderConstValue(reader());
-            if(pvalue && !attr.empty())
+
+            if(titleonproperty.find("ZPE")!=string::npos)
+            {
+              double energy;
+              stringstream ss((const char*)pvalue);
+              ss >> energy; //units are kJ/mol
+              const double CALSTOJOULES = 4.1816;
+              _pmol->SetEnergy(energy/CALSTOJOULES);
+            }
+            else
+            {
+              if(pvalue && !attr.empty())
               {
                 OBPairData *dp = new OBPairData;
                 dp->SetAttribute(attr);
@@ -377,6 +386,7 @@ namespace OpenBabel
                 dp->SetOrigin(fileformatInput);
                 _pmol->SetData(dp);
               }
+            }
             PropertyScalarsNeeded=0;
           }
       }
@@ -1207,7 +1217,7 @@ namespace OpenBabel
   }
 
 
-  void CMLFormat::WriteMetadataList()
+  void CMLFormat::WriteMetadataList(OBMol& mol)
   {
     static const xmlChar C_METADATALIST[] = "metadataList";
     static const xmlChar C_METADATA[]     = "metadata";
@@ -1216,7 +1226,20 @@ namespace OpenBabel
     static const xmlChar C_CONTENT[]      = "content";
 
     xmlTextWriterStartElement(writer(), C_METADATALIST);
-    xmlTextWriterWriteAttribute(writer(), C_TITLE, BAD_CAST "generated by OpenBabel");
+
+    if(mol.HasData(OBGenericDataType::CommentData))
+    {
+      OBCommentData *cd = (OBCommentData*)mol.GetData(OBGenericDataType::CommentData);
+      xmlTextWriterStartElement(writer(), C_METADATA);
+      xmlTextWriterWriteAttribute(writer(), C_NAME, BAD_CAST "dc:description");
+      xmlTextWriterWriteAttribute(writer(), C_CONTENT, BAD_CAST cd->GetData().c_str());
+      xmlTextWriterEndElement(writer());
+    }
+
+    xmlTextWriterStartElement(writer(), C_METADATA);
+    xmlTextWriterWriteAttribute(writer(), C_NAME, BAD_CAST "dc:source");
+    xmlTextWriterWriteAttribute(writer(), C_CONTENT, BAD_CAST "unknown");
+    xmlTextWriterEndElement(writer());
 
     xmlTextWriterStartElement(writer(), C_METADATA);
     xmlTextWriterWriteAttribute(writer(), C_NAME, BAD_CAST "dc:creator");
@@ -1225,16 +1248,6 @@ namespace OpenBabel
     xmlTextWriterWriteAttribute(writer(), C_CONTENT, BAD_CAST version.c_str());
     xmlTextWriterEndElement(writer());
 
-    xmlTextWriterStartElement(writer(), C_METADATA);
-    xmlTextWriterWriteAttribute(writer(), C_NAME, BAD_CAST "dc:description");
-    xmlTextWriterWriteAttribute(writer(), C_CONTENT, BAD_CAST "Conversion of legacy filetype to CML");
-    xmlTextWriterEndElement(writer());
-
-    xmlTextWriterStartElement(writer(), C_METADATA);
-    xmlTextWriterWriteAttribute(writer(), C_NAME, BAD_CAST "dc:type");
-    xmlTextWriterWriteAttribute(writer(), C_CONTENT, BAD_CAST "chemistry");
-    xmlTextWriterEndElement(writer());
-
     xmlTextWriterStartElement(writer(), C_METADATA);
     xmlTextWriterWriteAttribute(writer(), C_NAME, BAD_CAST "dc:contributor");
     xmlTextWriterWriteAttribute(writer(), C_CONTENT, BAD_CAST "unknown");
@@ -1245,11 +1258,11 @@ namespace OpenBabel
     xmlTextWriterWriteAttribute(writer(), C_CONTENT, BAD_CAST getTimestr().c_str());
     xmlTextWriterEndElement(writer());
 
-    xmlTextWriterStartElement(writer(), C_METADATA);
+/*    xmlTextWriterStartElement(writer(), C_METADATA);
     xmlTextWriterWriteAttribute(writer(), C_NAME, BAD_CAST "cmlm:structure");
     xmlTextWriterWriteAttribute(writer(), C_CONTENT, BAD_CAST "yes");
     xmlTextWriterEndElement(writer());
-
+*/
     xmlTextWriterEndElement(writer());
   }
 
@@ -1305,6 +1318,7 @@ namespace OpenBabel
     static const xmlChar C_ATOMREF2[]         = "atomRef2";
     static const xmlChar C_ORDER[]            = "order";
     static const xmlChar C_ATOMREFS4[]        = "atomRefs4";
+    static const xmlChar C_DESCRIPTION[]      = "description";
     /* defined in other functions
        static const xmlChar C_FORMULA[] = "formula";
        static const xmlChar C_CONCISE[] = "concise";
@@ -1397,7 +1411,7 @@ namespace OpenBabel
         UseHydrogenCount=true;
       }
 
-    bool UseFormulaWithNoBonds=true;
+    bool UseFormulaWithNoBonds=false; //before 2.3.1 was true;
 
     int dim = mol.GetDimension();
 
@@ -1422,7 +1436,7 @@ namespace OpenBabel
         if(!isalpha(name[0])) //since ids have to start with a letter, add "id" to those that don't...
           name = "id" + name;
         xmlTextWriterWriteAttribute(writer(), C_ID, BAD_CAST name.c_str());
-        if(!isalpha(*id)) //...and write <name> orig title </name>
+        if(!isalpha(name[0])) //...and write <name> orig title </name>
           {
             xmlTextWriterStartElementNS(writer(), prefix, C_NAME, NULL);
             xmlTextWriterWriteFormatString(writer(),"%s", id);
@@ -1431,7 +1445,7 @@ namespace OpenBabel
       }
 
     if(_pxmlConv->IsOption("m") && _pxmlConv->GetOutputIndex()==1) //only on first molecule
-      WriteMetadataList();
+      WriteMetadataList(mol);
 
     pUnitCell = NULL;
     if (!cml1 && mol.HasData(OBGenericDataType::UnitCell))
@@ -2137,7 +2151,7 @@ namespace OpenBabel
           }
       }
 
-    static const double CALSTOJOULES = 4.1816;
+    static const double CALSTOJOULES = 4.184;
     //Energy is output when it is not zero
     //This is the molecular energy, probably originally in Hartrees,
     // stored in OB as kcal/mol, but output here in kJ/mol
@@ -2308,7 +2322,7 @@ namespace OpenBabel
     xmlTextWriterStartElementNS(writer(), prefix, C_ARRAY, NULL);
     xmlTextWriterWriteFormatAttribute(writer(), C_UNITS,"%s","cm-1");
     const double WAVENUM_TO_GHZ=30.0;
-    for(int i=0; i<3; ++i)
+    for(int i=0; i<rd->GetRotConsts().size(); ++i)
       if(rd->GetRotConsts()[i]!=0.0)
         xmlTextWriterWriteFormatString(writer(),"%.3lf ", rd->GetRotConsts()[i]/WAVENUM_TO_GHZ);
     xmlTextWriterEndElement(writer());//array
@@ -2334,6 +2348,7 @@ namespace OpenBabel
     static const xmlChar C_UNITS[]        = "units";
     static const xmlChar C_TITLE[]        = "title";
     static const xmlChar C_CONVENTION[]   = "convention";
+    static const xmlChar C_ZPEADDED[]   = "zeroPointVibEnergyAdded";
 
     xmlTextWriterStartElementNS(writer(), prefix, C_PROPERTY, NULL);
     xmlTextWriterWriteFormatAttribute(writer(), C_TITLE,"%s",title);
@@ -2343,7 +2358,11 @@ namespace OpenBabel
     if(units)
       xmlTextWriterWriteFormatAttribute(writer(), C_UNITS,"%s",units);
     if(convention)
+    {
       xmlTextWriterWriteFormatAttribute(writer(), C_CONVENTION,"%s",convention);
+      if(strcmp(convention, "computational")==0)
+        xmlTextWriterWriteFormatAttribute(writer(), C_ZPEADDED,"%s","false");
+    }
     xmlTextWriterWriteFormatString(writer(),"%.2lf ", value);
     xmlTextWriterEndElement(writer());//scalar
     xmlTextWriterEndElement(writer());//property
