@@ -1,16 +1,17 @@
 /**********************************************************************
 generic.cpp - Handle OBGenericData classes.
- 
+
 Copyright (C) 1998-2001 by OpenEye Scientific Software, Inc.
 Some portions Copyright (C) 2001-2006 by Geoffrey R. Hutchison
- 
+Some portions Copyright (C) 2010 by David Lonie
+
 This file is part of the Open Babel project.
-For more information, see <http://openbabel.sourceforge.net/>
- 
+For more information, see <http://openbabel.org/>
+
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation version 2 of the License.
- 
+
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -45,13 +46,13 @@ namespace OpenBabel
 
       For more information on currently supported types, please see
       the developer wiki:
-      http://openbabel.sourceforge.net/wiki/Generic_Data
+      http://openbabel.org/wiki/Generic_Data
 
-      For your own custom data, either define a custom subclass using 
-      an id from the OBGenericDataType::CustomData0 to 
+      For your own custom data, either define a custom subclass using
+      an id from the OBGenericDataType::CustomData0 to
       OBGenericDataType::CustomData15 slots,
       or store your data as a string and use OBPairData for key/value access.
-      The latter is <strong>highly</strong> recommended for various text 
+      The latter is <strong>highly</strong> recommended for various text
       descriptors
       e.g., in QSAR, atom or bond labels, or other textual data.
 
@@ -67,7 +68,7 @@ namespace OpenBabel
       in OBBase, allowing you to separate data read in from a file,
       added by a user, or assigned by Open Babel internally.
 
-      While the library and import routines will set DataOrigin correctly, 
+      While the library and import routines will set DataOrigin correctly,
       you should try to annotate data added by your code. Typically this would
       either be userAdded or external. The former refers to something the
       user requested as an annotation, while the latter refers to annotations
@@ -98,7 +99,7 @@ namespace OpenBabel
          }
       @endcode
 
-      Similar code also works for OBGenericData stored in an OBAtom or 
+      Similar code also works for OBGenericData stored in an OBAtom or
       OBBond or OBResidue. These examples show use of DataOrigin outside
       of the Open Babel library.
 
@@ -126,27 +127,27 @@ namespace OpenBabel
       }
       @endcode
 
-      When designing a class derived from OBGenericData you must add a 
-      Clone() function. For classes used with OBMol this is used when 
+      When designing a class derived from OBGenericData you must add a
+      Clone() function. For classes used with OBMol this is used when
       an OBMol object is copied. If your class member variables contain
       pointers to atoms or bonds then it will be necessary to ensure
       that these are updated in Clone() to refer to the new molecule. Without
-      these and similar pointers it is more likely that the very simple 
+      these and similar pointers it is more likely that the very simple
       clone function
       @code
       virtual OBGenericData* Clone(OBBase* parent) const
          {return new MyNewClass(*this);}
       @endcode
-      and the compiler generated copy constructor would be sufficient. 
+      and the compiler generated copy constructor would be sufficient.
 
       It is recommended that, if possible, OBGenericData classes do not
       store atom and bond pointers. Using atom and bond indices instead
-      would allow the simple version of Clone() above. See 
+      would allow the simple version of Clone() above. See
       OBRotameterData::Clone for an example of a more complicated version.
-      For classes which are not intended to support copying, Clone() can 
-      return NULL 
+      For classes which are not intended to support copying, Clone() can
+      return NULL
       @code
-      virtual OBGenericData* Clone(OBBase* parent) const 
+      virtual OBGenericData* Clone(OBBase* parent) const
          {return NULL;}
       @endcode
       Clone() is a pure virtual function so that you need to decide what
@@ -247,21 +248,19 @@ namespace OpenBabel
   //
   OBUnitCell::OBUnitCell():
     OBGenericData("UnitCell", OBGenericDataType::UnitCell),
-    _a(0.0), _b(0.0), _c(0.0), _alpha(0.0), _beta(0.0), _gamma(0.0),
-    _spaceGroup( NULL ), _lattice(Undefined)
-  {  
-    // We should default to P1 space group unless we know differently
-    SetSpaceGroup(1);
-  }
+    _mOrtho(matrix3x3()),
+    _mOrient(matrix3x3()),
+    _offset(vector3()),
+    _spaceGroupName(""), _spaceGroup( NULL ),
+    _lattice(Undefined)
+  {  }
 
   OBUnitCell::OBUnitCell(const OBUnitCell &src) :
     OBGenericData("UnitCell", OBGenericDataType::UnitCell),
-    _a(src._a), _b(src._b), _c(src._c), 
-    _alpha(src._alpha), _beta(src._beta), _gamma(src._gamma),
+    _mOrtho(src._mOrtho),
+    _mOrient(src._mOrient),
     _offset(src._offset),
-    _v1(src._v1), _v2(src._v2), _v3(src._v3),
-    _spaceGroupName(src._spaceGroupName),
-    _spaceGroup(src._spaceGroup),
+    _spaceGroupName(src._spaceGroupName), _spaceGroup(src._spaceGroup),
     _lattice(src._lattice)
   {  }
 
@@ -270,18 +269,10 @@ namespace OpenBabel
     if(this == &src)
       return(*this);
 
-    _a = src._a;
-    _b = src._b;
-    _c = src._c;
-    _alpha = src._alpha;
-    _beta = src._beta;
-    _gamma = src._gamma;
+    _mOrtho = src._mOrtho;
+    _mOrient = src._mOrient;
     _offset = src._offset;
 
-    _v1 = src._v1;
-    _v2 = src._v2;
-    _v3 = src._v3;
-
     _spaceGroup = src._spaceGroup;
     _spaceGroupName = src._spaceGroupName;
     _lattice = src._lattice;
@@ -289,50 +280,42 @@ namespace OpenBabel
     return(*this);
   }
 
-	/*!
-  ** The angles and lengths of the unitcell will be calculated from the
-  ** vectors @p v1, @p v2 and @p v3. Those vectors will as well be
-  ** stored internally.
-  **Implements <a href="http://qsar.sourceforge.net/dicts/blue-obelisk/index.xhtml#convertCartesianIntoNotionalCoordinates">blue-obelisk:convertCartesianIntoNotionalCoordinates</a>
-  **\brief Sets the vectors, angles and lengths of the unitcell
-  **\param v1 The x-vector
-  **\param v2 The y-vector
-  **\param v3 The z-vector
-  **\see OBUnitCell::GetCellVectors
-  */
+  //! Implements <a href="http://qsar.sourceforge.net/dicts/blue-obelisk/index.xhtml#calculateOrthogonalisationMatrix">blue-obelisk:calculateOrthogonalisationMatrix</a>
+  void OBUnitCell::SetData(const double a, const double b, const double c,
+                           const double alpha, const double beta, const double gamma)
+  {
+    _mOrtho.FillOrth(alpha, beta, gamma, a, b, c);
+    _mOrient = matrix3x3(1);
+    _spaceGroup = NULL;
+    _spaceGroupName = "";
+    _lattice = OBUnitCell::Undefined;
+  }
+
+  //! Implements <a href="http://qsar.sourceforge.net/dicts/blue-obelisk/index.xhtml#calculateOrthogonalisationMatrix">blue-obelisk:calculateOrthogonalisationMatrix</a>
   void OBUnitCell::SetData(const vector3 v1, const vector3 v2, const vector3 v3)
   {
-    _v1 = v1;
-    _v2 = v2;
-    _v3 = v3;
+    matrix3x3 m (v1, v2, v3);
+    _mOrtho.FillOrth(vectorAngle(v2,v3), // alpha
+                     vectorAngle(v1,v3), // beta
+                     vectorAngle(v1,v2), // gamma
+                     v1.length(),        // a
+                     v2.length(),        // b
+                     v3.length());       // c
+    _mOrient = m.transpose() * _mOrtho.inverse();
+    _spaceGroup = NULL;
+    _spaceGroupName = "";
+    _lattice = OBUnitCell::Undefined;
+  }
 
-    _a = _v1.length();
-    _b = _v2.length();
-    _c = _v3.length();
+  //! Implements <a href="http://qsar.sourceforge.net/dicts/blue-obelisk/index.xhtml#calculateOrthogonalisationMatrix">blue-obelisk:calculateOrthogonalisationMatrix</a>
+  void OBUnitCell::SetData(const matrix3x3 m)
+  {
+    SetData(m.GetRow(0), m.GetRow(1), m.GetRow(2));
+  }
 
-    // For PR#1961604 -- somewhat contrived example
-    if (IsNearZero(_a) && !IsNearZero(_c)) {
-      _v1 = _v3; // we'll reset _v3 below
-      _a = _c;
-      _c = 0.0;
-    }
-
-    // Sanity checks for 1D or 2D translation
-    if (IsNearZero(_b)) { // 1D
-      _v2.Set(v1.y(), -v1.x(), v1.z()); // rotate base vector by 90 degrees
-      _b = 999.999;
-      _v2 = _b * _v2.normalize(); // set to a large displacement
-    }
-
-    if (IsNearZero(_c)) { // 2D or 1D
-      _v3 = cross(_v1, _v2);
-      _c = 999.999;
-      _v3 = _c * _v3.normalize(); // set to a large displacement
-    }
-    
-    _alpha = vectorAngle(_v2, _v3);
-    _beta =  vectorAngle(_v1, _v3);
-    _gamma = vectorAngle(_v1, _v2);
+  void OBUnitCell::SetOffset(const vector3 v1)
+  {
+    _offset = v1;
   }
 
   //! Implements <a href="http://qsar.sourceforge.net/dicts/blue-obelisk/index.xhtml#convertNotionalIntoCartesianCoordinates">blue-obelisk:convertNotionalIntoCartesianCoordinates</a>
@@ -341,152 +324,112 @@ namespace OpenBabel
     vector<vector3> v;
     v.reserve(3);
 
-    // no unit cell vectors
-    if (IsNegligible(_v1.length(), 1.0, 1.0e-9) &&
-        IsNegligible(_v2.length(), 1.0, 1.0e-9) &&
-        IsNegligible(_v3.length(), 1.0, 1.0e-9))
-      {
-        vector3 temp;
-        matrix3x3 m = GetOrthoMatrix();
+    matrix3x3 m = GetCellMatrix();
 
-        temp = vector3(1.0, 0.0, 0.0);
-        v.push_back(m * temp);
-        temp = vector3(0.0, 1.0, 0.0);
-        v.push_back(m * temp);
-        temp = vector3(0.0, 0.0, 1.0);
-        v.push_back(m * temp);
-      }
-    else
-      {
-        v.push_back(_v1);
-        // we set these above in case we had a 1D or 2D translation vector system
-        if (fabs(_b - 999.999) > 1.0e-1)
-          v.push_back(_v2);
-        if (fabs(_c - 999.999) > 1.0e-1)
-          v.push_back(_v3);
-      }
+    v.push_back(m.GetRow(0));
+    v.push_back(m.GetRow(1));
+    v.push_back(m.GetRow(2));
 
     return v;
   }
 
   matrix3x3 OBUnitCell::GetCellMatrix()
   {
-    matrix3x3 m;
-
-    if (IsNegligible(_v1.length(), 1.0, 1.0e-9) &&
-        IsNegligible(_v2.length(), 1.0, 1.0e-9) &&
-        IsNegligible(_v3.length(), 1.0, 1.0e-9))
-      {
-        m = GetOrthoMatrix();
-      }
-    else
-      {
-        vector3 v1, v2, v3;
-        v1 = _v1;
-        v2 = _v2;
-        v3 = _v3;
-        m = matrix3x3(v1,v2,v3);
-      }
-    return m;
+    return (_mOrient * _mOrtho).transpose();
   }
 
-  // Convert from fractional to Cartesian
-  //! Implements <a href="http://qsar.sourceforge.net/dicts/blue-obelisk/index.xhtml#calculateOrthogonalisationMatrix">blue-obelisk:calculateOrthogonalisationMatrix</a>
   matrix3x3 OBUnitCell::GetOrthoMatrix()
   {
-    matrix3x3 m;
-    if (IsNearZero(_c) || IsNearZero(_b)) {
-      // 1D or 2D unit cell
-    }
+    return _mOrtho;
+  }
 
-    // already here, let's not duplicate the work
-    m.FillOrth(_alpha, _beta, _gamma, _a, _b, _c);
-
-    return m;
+  matrix3x3 OBUnitCell::GetOrientationMatrix()
+  {
+    return _mOrient;
   }
 
-  // Based on code in PyMMLib: http://pymmlib.sf.net/
-  //! Matrix to convert from Cartesian to fractional
-  //! Implements <a href="http://qsar.sourceforge.net/dicts/blue-obelisk/index.xhtml#convertCartesianIntoFractionalCoordinates">blue-obelisk:convertCartesianIntoFractionalCoordinates</a> 
   matrix3x3 OBUnitCell::GetFractionalMatrix()
   {
-    matrix3x3 m;
-    double sinAlpha, sinBeta, sinGamma;
-    double cosAlpha, cosBeta, cosGamma;
-    double v;
+    return _mOrtho.inverse();
+  }
 
-    sinAlpha = sin(_alpha * DEG_TO_RAD);
-    sinBeta = sin(_beta * DEG_TO_RAD);
-    sinGamma = sin(_gamma * DEG_TO_RAD);
-    cosAlpha = cos(_alpha * DEG_TO_RAD);
-    cosBeta = cos(_beta * DEG_TO_RAD);
-    cosGamma = cos(_gamma * DEG_TO_RAD);
+  vector3 OBUnitCell::FractionalToCartesian(vector3 frac)
+  {
+    return _mOrient * _mOrtho * frac + _offset;
+  }
 
-    v = sqrt(1 - SQUARE(cosAlpha) - SQUARE(cosBeta) - SQUARE(cosGamma) +
-             2 * cosAlpha*cosBeta*cosGamma);
+  vector3 OBUnitCell::CartesianToFractional(vector3 cart)
+  {
+    return _mOrtho.inverse() * _mOrient.inverse() * (cart - _offset);
+  }
 
-    m.Set(0,0,  1.0 / _a);
-    m.Set(0,1,  -cosGamma / (_a * sinGamma) );
-    m.Set(0,2,  (cosGamma * cosAlpha - cosBeta) / (_a * v * sinGamma) );
-    m.Set(1,0,  0.0);
-    m.Set(1,1,  1.0 / (_b * sinGamma) );
-    m.Set(1,2,  (cosGamma * cosBeta - cosAlpha) / (_b * v * sinGamma) );
-    m.Set(2,0,  0.0);
-    m.Set(2,1,  0.0);
-    m.Set(2,2,  sinGamma / (_c * v) );
+  vector3 OBUnitCell::WrapCartesianCoordinate(vector3 cart)
+  {
+    vector3 v = CartesianToFractional(cart);
+    v = WrapFractionalCoordinate(v);
+    return FractionalToCartesian(v);
+  }
 
-    return m;
+  vector3 OBUnitCell::WrapFractionalCoordinate(vector3 frac)
+  {
+    double x = fmod(frac.x(), 1);
+    double y = fmod(frac.y(), 1);
+    double z = fmod(frac.z(), 1);
+    if (x < 0) x += 1;
+    if (y < 0) y += 1;
+    if (z < 0) z += 1;
+    return vector3(x, y, z);
   }
 
   OBUnitCell::LatticeType OBUnitCell::GetLatticeType( int spacegroup )
   {
-	  //	1-2 	Triclinic
-	  //	3-15 	Monoclinic
-	  //	16-74	Orthorhombic
-	  //	75-142 	Tetragonal
-	  //	143-167 Rhombohedral
-	  //	168-194 Hexagonal
-	  //	195-230 Cubic
+    //	1-2 	Triclinic
+    //	3-15 	Monoclinic
+    //	16-74	Orthorhombic
+    //	75-142 	Tetragonal
+    //	143-167 Rhombohedral
+    //	168-194 Hexagonal
+    //	195-230 Cubic
 
-      if ( spacegroup == 0  && _spaceGroup)
-          spacegroup = _spaceGroup->GetId();
-	  
-	  if ( spacegroup <= 0 )
-		  return OBUnitCell::Undefined;
+    if ( spacegroup == 0  && _spaceGroup)
+      spacegroup = _spaceGroup->GetId();
 
-	  else if ( spacegroup == 1 ||
+    if ( spacegroup <= 0 )
+      return OBUnitCell::Undefined;
+
+    else if ( spacegroup == 1 ||
               spacegroup == 2 )
-		  return OBUnitCell::Triclinic;
-	  
-	  else if ( spacegroup >= 3 &&
-              spacegroup <= 15 )
-		  return OBUnitCell::Monoclinic;
-	  
-	  else if ( spacegroup >= 16 &&
-              spacegroup <= 74 )
-		  return OBUnitCell::Orthorhombic;
-	  
-	  else if ( spacegroup >= 75 &&
-              spacegroup <= 142 )
-		  return OBUnitCell::Tetragonal;
-	  
-	  else if ( spacegroup >= 143 &&
-              spacegroup <= 167 )
-		  return OBUnitCell::Rhombohedral;
-	  
-	  else if ( spacegroup >= 168 &&
-              spacegroup <= 194 )
-		  return OBUnitCell::Hexagonal;
-	  
-	  else if ( spacegroup >= 195 &&
-              spacegroup <= 230 )
-		  return OBUnitCell::Cubic;
+      return OBUnitCell::Triclinic;
 
-	  //just to be extra sure
-	  else // ( spacegroup > 230 )
-		  return OBUnitCell::Undefined;
+    else if ( spacegroup >= 3 &&
+              spacegroup <= 15 )
+      return OBUnitCell::Monoclinic;
+
+    else if ( spacegroup >= 16 &&
+              spacegroup <= 74 )
+      return OBUnitCell::Orthorhombic;
+
+    else if ( spacegroup >= 75 &&
+              spacegroup <= 142 )
+      return OBUnitCell::Tetragonal;
+
+    else if ( spacegroup >= 143 &&
+              spacegroup <= 167 )
+      return OBUnitCell::Rhombohedral;
+
+    else if ( spacegroup >= 168 &&
+              spacegroup <= 194 )
+      return OBUnitCell::Hexagonal;
+
+    else if ( spacegroup >= 195 &&
+              spacegroup <= 230 )
+      return OBUnitCell::Cubic;
+
+    //just to be extra sure
+    else // ( spacegroup > 230 )
+      return OBUnitCell::Undefined;
   }
-  
+
   OBUnitCell::LatticeType OBUnitCell::GetLatticeType()
   {
     if (_lattice != Undefined)
@@ -494,32 +437,39 @@ namespace OpenBabel
     else if (_spaceGroup != NULL)
       return GetLatticeType(_spaceGroup->GetId());
 
+    double a = GetA();
+    double b = GetB();
+    double c = GetC();
+    double alpha = GetAlpha();
+    double beta  = GetBeta();
+    double gamma = GetGamma();
+
     unsigned int rightAngles = 0;
-    if (IsApprox(_alpha, 90.0, 1.0e-3)) rightAngles++;
-    if (IsApprox(_beta,  90.0, 1.0e-3)) rightAngles++;
-    if (IsApprox(_gamma, 90.0, 1.0e-3)) rightAngles++;
+    if (IsApprox(alpha, 90.0, 1.0e-3)) rightAngles++;
+    if (IsApprox(beta,  90.0, 1.0e-3)) rightAngles++;
+    if (IsApprox(gamma, 90.0, 1.0e-3)) rightAngles++;
 
     switch (rightAngles)
       {
       case 3:
-        if (IsApprox(_a, _b, 1.0e-4) && IsApprox(_b, _c, 1.0e-4))
+        if (IsApprox(a, b, 1.0e-4) && IsApprox(b, c, 1.0e-4))
           _lattice = Cubic;
-        else if (IsApprox(_a, _b, 1.0e-4) || IsApprox(_b, _c, 1.0e-4))
+        else if (IsApprox(a, b, 1.0e-4) || IsApprox(b, c, 1.0e-4))
           _lattice = Tetragonal;
         else
           _lattice = Orthorhombic;
         break;
       case 2:
-        if ( (IsApprox(_alpha, 120.0, 1.0e-3) 
-              || IsApprox(_beta, 120.0, 1.0e-3) 
-              || IsApprox(_gamma, 120.0f, 1.0e-3))
-             && (IsApprox(_a, _b, 1.0e-4) || IsApprox(_b, _c, 1.0e-4)) )
+        if ( (IsApprox(alpha, 120.0, 1.0e-3)
+              || IsApprox(beta, 120.0, 1.0e-3)
+              || IsApprox(gamma, 120.0f, 1.0e-3))
+             && (IsApprox(a, b, 1.0e-4) || IsApprox(b, c, 1.0e-4)) )
           _lattice = Hexagonal;
         else
           _lattice = Monoclinic;
         break;
       default:
-        if (IsApprox(_a, _b, 1.0e-4) && IsApprox(_b, _c, 1.0e-4))
+        if (IsApprox(a, b, 1.0e-4) && IsApprox(b, c, 1.0e-4))
           _lattice = Rhombohedral;
         else
           _lattice = Triclinic;
@@ -530,25 +480,25 @@ namespace OpenBabel
 
   int OBUnitCell::GetSpaceGroupNumber( std::string name)
   {
-    static const char * const spacegroups[] = { 
-      "P1", "P-1", "P2", "P2(1)", "C2", "Pm", "Pc", "Cm", "Cc", "P2/m", 
-      "P2(1)/m", "C2/m", "P2/c", "P2(1)/c", "C2/c", "P222", "P222(1)", 
-      "P2(1)2(1)2", "P2(1)2(1)2(1)", "C222(1)", "C222", "F222", "I222", 
-      "I2(1)2(1)2(1)", "Pmm2", "Pmc2(1)", "Pcc2", "Pma2", "Pca2(1)", "Pnc2", 
-      "Pmn2(1)", "Pba2", "Pna2(1)", "Pnn2", "Cmm2", "Cmc2(1)", "Ccc2", "Amm2", 
-      "Abm2", "Ama2", "Aba2", "Fmm2", "Fdd2", "Imm2", "Iba2", "Ima2", "Pmmm", 
-      "Pnnn", "Pccm", "Pban", "Pmma", "Pnna", "Pmna", "Pcca", "Pbam", "Pccn", 
-      "Pbcm", "Pnnm", "Pmmn", "Pbcn", "Pbca", "Pnma", "Cmcm", "Cmca", "Cmmm", 
-      "Cccm", "Cmma", "Ccca", "Fmmm", "Fddd", "Immm", "Ibam", "Ibca", "Imma", 
-      "P4", "P4(1)", "P4(2)", "P4(3)", "I4", "I4(1)", "P-4", "I-4", "P4/m", 
-      "P4(2)/m", "P4/n", "P4(2)/n", "I4/m", "I4(1)/a", "P422", "P42(1)2", 
+    static const char * const spacegroups[] = {
+      "P1", "P-1", "P2", "P2(1)", "C2", "Pm", "Pc", "Cm", "Cc", "P2/m",
+      "P2(1)/m", "C2/m", "P2/c", "P2(1)/c", "C2/c", "P222", "P222(1)",
+      "P2(1)2(1)2", "P2(1)2(1)2(1)", "C222(1)", "C222", "F222", "I222",
+      "I2(1)2(1)2(1)", "Pmm2", "Pmc2(1)", "Pcc2", "Pma2", "Pca2(1)", "Pnc2",
+      "Pmn2(1)", "Pba2", "Pna2(1)", "Pnn2", "Cmm2", "Cmc2(1)", "Ccc2", "Amm2",
+      "Abm2", "Ama2", "Aba2", "Fmm2", "Fdd2", "Imm2", "Iba2", "Ima2", "Pmmm",
+      "Pnnn", "Pccm", "Pban", "Pmma", "Pnna", "Pmna", "Pcca", "Pbam", "Pccn",
+      "Pbcm", "Pnnm", "Pmmn", "Pbcn", "Pbca", "Pnma", "Cmcm", "Cmca", "Cmmm",
+      "Cccm", "Cmma", "Ccca", "Fmmm", "Fddd", "Immm", "Ibam", "Ibca", "Imma",
+      "P4", "P4(1)", "P4(2)", "P4(3)", "I4", "I4(1)", "P-4", "I-4", "P4/m",
+      "P4(2)/m", "P4/n", "P4(2)/n", "I4/m", "I4(1)/a", "P422", "P42(1)2",
       "P4(1)22", "P4(1)2(1)2", "P4(2)22", "P4(2)2(1)2", "P4(3)22", "P4(3)2(1)2",
       "I422", "I4(1)22", "P4mm", "P4bm", "P4(2)cm", "P4(2)nm", "P4cc", "P4nc",
-      "P4(2)mc", "P4(2)bc", "I4mm", "I4cm", "I4(1)md", "I4(1)cd", "P-42m", 
-      "P-42c", "P-42(1)m", "P-42(1)c", "P-4m2", "P-4c2", "P-4b2", "P-4n2", 
+      "P4(2)mc", "P4(2)bc", "I4mm", "I4cm", "I4(1)md", "I4(1)cd", "P-42m",
+      "P-42c", "P-42(1)m", "P-42(1)c", "P-4m2", "P-4c2", "P-4b2", "P-4n2",
       "I-4m2", "I-4c2", "I-42m", "I-42d", "P4/mmm", "P4/mcc", "P4/nbm",
-      "P4/nnc", "P4/mbm", "P4/mnc", "P4/nmm", "P4/ncc", "P4(2)/mmc", 
-      "P4(2)/mcm", "P4(2)/nbc", "P4(2)/nnm", "P4(2)/mbc", "P4(2)/mnm", 
+      "P4/nnc", "P4/mbm", "P4/mnc", "P4/nmm", "P4/ncc", "P4(2)/mmc",
+      "P4(2)/mcm", "P4(2)/nbc", "P4(2)/nnm", "P4(2)/mbc", "P4(2)/mnm",
       "P4(2)/nmc", "P4(2)/ncm", "I4/mmm", "I4/mcm", "I4(1)/amd", "I4(1)/acd",
       "P3", "P3(1)", "P3(2)", "R3", "P-3", "R-3", "P312", "P321", "P3(1)12",
       "P3(1)21", "P3(2)12", "P3(2)21", "R32", "P3m1", "P31m", "P3c1", "P31c",
@@ -560,7 +510,7 @@ namespace OpenBabel
       "P2(1)3", "I2(1)3", "Pm-3", "Pn-3", "Fm-3", "Fd-3", "Im-3", "Pa-3",
       "Ia-3", "P432", "P4(2)32", "F432", "F4(1)32", "I432", "P4(3)32",
       "P4(1)32", "I4(1)32", "P-43m", "F4-3m", "I-43m", "P-43n", "F-43c",
-      "I-43d", "Pm-3m", "Pn-3n", "Pm-3n", "Pn-3m", "Fm-3m", "Fm-3c", 
+      "I-43d", "Pm-3m", "Pn-3n", "Pm-3n", "Pn-3m", "Fm-3m", "Fm-3c",
       "Fd-3m", "Fd-3c", "Im-3m", "Ia-3d"
     };
 
@@ -604,7 +554,7 @@ namespace OpenBabel
   void OBUnitCell::FillUnitCell(OBMol *mol)
   {
     const SpaceGroup *sg = GetSpaceGroup(); // the actual space group and transformations for this unit cell
-    
+
     // For each atom, we loop through: convert the coords back to inverse space, apply the transformations and create new atoms
     vector3 uniqueV, newV, updatedCoordinate;
     list<vector3> transformedVectors; // list of symmetry-defined copies of the atom
@@ -619,10 +569,10 @@ namespace OpenBabel
     list<OBAtom*>::iterator i;
     for (i = atoms.begin(); i != atoms.end(); ++i) {
       uniqueV = (*i)->GetVector();
-      uniqueV *= GetFractionalMatrix();
+      uniqueV = CartesianToFractional(uniqueV);
       uniqueV = transformedFractionalCoordinate(uniqueV);
       coordinates.push_back(uniqueV);
-  
+
       transformedVectors = sg->Transform(uniqueV);
       for (transformIterator = transformedVectors.begin();
            transformIterator != transformedVectors.end(); ++transformIterator) {
@@ -641,73 +591,67 @@ namespace OpenBabel
         }
         if (foundDuplicate)
           continue;
-        
+
         coordinates.push_back(updatedCoordinate); // make sure to check the new atom for dupes
         newAtom = mol->NewAtom();
         newAtom->Duplicate(*i);
-        newAtom->SetVector(GetOrthoMatrix() * updatedCoordinate);
+        newAtom->SetVector(FractionalToCartesian(updatedCoordinate));
       } // end loop of transformed atoms
-      (*i)->SetVector(GetOrthoMatrix() * uniqueV); // move the atom back into the unit cell
+      (*i)->SetVector(FractionalToCartesian(uniqueV)); // move the atom back into the unit cell
     } // end loop of atoms
 
     SetSpaceGroup(1); // We've now applied the symmetry, so we should act like a P1 unit cell
   }
-  
+
+  double OBUnitCell::GetA()
+  {
+    return _mOrtho.GetColumn(0).length();
+  }
+
+  double OBUnitCell::GetB()
+  {
+    return _mOrtho.GetColumn(1).length();
+  }
+
+  double OBUnitCell::GetC()
+  {
+    return _mOrtho.GetColumn(2).length();
+  }
+
+  double OBUnitCell::GetAlpha()
+  {
+    return vectorAngle(_mOrtho.GetColumn(1), _mOrtho.GetColumn(2));
+  }
+
+  double OBUnitCell::GetBeta()
+  {
+    return vectorAngle(_mOrtho.GetColumn(0), _mOrtho.GetColumn(2));
+  }
+
+  double OBUnitCell::GetGamma()
+  {
+    return vectorAngle(_mOrtho.GetColumn(0), _mOrtho.GetColumn(1));
+  }
+
+  vector3 OBUnitCell::GetOffset()
+  {
+    return _offset;
+  }
+
   double OBUnitCell::GetCellVolume()
   {
-    double result = 0.0;
-    
-    switch ( GetLatticeType() )
-      {
-      case Triclinic:
-        result = _a * _b * _c 
-          * sqrt(1
-                 - SQUARE(cos( _alpha * DEG_TO_RAD ))
-                 - SQUARE(cos( _beta * DEG_TO_RAD ))
-                 - SQUARE(cos( _gamma * DEG_TO_RAD ))
-                 + 2 * cos( _alpha * DEG_TO_RAD ) * cos( _beta * DEG_TO_RAD ) * cos( _gamma * DEG_TO_RAD )
-                 );
-        break;
-      case Monoclinic:
-        result = _a * _b * _c * sin( _beta * DEG_TO_RAD );
-        break;
-      case Orthorhombic:
-        result = _a * _b * _c;
-        break;
-      case Tetragonal:
-        result = _a * _a * _c;
-        break;
-      case Rhombohedral:
-        result = _a * _a * _a
-          * sqrt(1
-                 - SQUARE(cos( _alpha * DEG_TO_RAD ))
-                 - SQUARE(cos( _beta * DEG_TO_RAD ))
-                 - SQUARE(cos( _gamma * DEG_TO_RAD ))
-                 + 2 * cos( _alpha * DEG_TO_RAD ) * cos( _beta * DEG_TO_RAD ) * cos( _gamma * DEG_TO_RAD )
-                 );
-        break;
-      case Hexagonal:
-        result = pow( 3.0, 0.333333333 ) * _a * _a * _c / 2;
-        break;
-      case Cubic:
-        result = _a * _a * _a;
-        break;
-      default:
-        result = 0.0;
-      }
-    
-    return result;
+    return fabs(GetCellMatrix().determinant());
   }
-  
+
   //
   // member functions for OBSymmetryData class
   //
-  OBSymmetryData::OBSymmetryData(): 
+  OBSymmetryData::OBSymmetryData():
     OBGenericData("Symmetry", OBGenericDataType::SymmetryData)
   { }
 
   OBSymmetryData::OBSymmetryData(const OBSymmetryData &src) :
-    OBGenericData(src._attr, src._type, src._source), 
+    OBGenericData(src._attr, src._type, src._source),
     _pointGroup(src._pointGroup), _spaceGroup(src._spaceGroup)
   {  }
 
@@ -739,7 +683,7 @@ namespace OpenBabel
   {
     if(this == &src)
       return(*this);
-    
+
     _source = src._source;
 
     _vDimension = src._vDimension;
@@ -851,7 +795,7 @@ namespace OpenBabel
   **\brief Angle default constructor
   */
   OBAngle::OBAngle():
-    _vertex(NULL), _termini(NULL, NULL), _radians(0.0)
+    _vertex((OBAtom *)NULL), _termini((OBAtom *)NULL, (OBAtom *)NULL), _radians(0.0)
   {  }
 
   /*!
@@ -926,7 +870,7 @@ namespace OpenBabel
 
   /*!
   **\brief Retrieves the 3 atom pointer for the angle (vertex first)
-  **\return triple of OBAtom pointers 
+  **\return triple of OBAtom pointers
   */
   triple<OBAtom*,OBAtom*,OBAtom*> OBAngle::GetAtoms()
   {
@@ -1024,7 +968,7 @@ namespace OpenBabel
       return(false);
 
     vector<OBAngle>::iterator angle;
-    
+
     angles.clear();
     angles.resize(_angles.size());
 
@@ -1037,10 +981,10 @@ namespace OpenBabel
         angles[ct][1] = angle->_termini.first->GetIdx() - 1;
         angles[ct][2] = angle->_termini.second->GetIdx() - 1;
       }
- 
+
     return(true);
   }
-  
+
   /*!
   **\brief Fills an array with the indices of the atoms in the angle (vertex first)
   **\param angles pointer to the pointer to an array of angles atom indices
@@ -1066,7 +1010,7 @@ namespace OpenBabel
       }
     return (unsigned int)_angles.size();
   }
-  
+
   //
   //member functions for OBAngleData class - stores OBAngle set
   //
@@ -1174,7 +1118,7 @@ namespace OpenBabel
 
   /*!
   **\brief determines if torsion has only protons on either the a or d end
-  **\return boolean 
+  **\return boolean
   */
   bool OBTorsion::IsProtonRotor()
   {
@@ -1321,7 +1265,7 @@ namespace OpenBabel
     case input: _atom4refs = atom4refs;break;
     case output:_atom4refo = atom4refs;break;
     case calcvolume:_atom4refc = atom4refs;break;
-    default: 
+    default:
       obErrorLog.ThrowError(__FUNCTION__, "AtomRefType called is invalid", obDebug);
       return(false);
     }
@@ -1338,7 +1282,7 @@ namespace OpenBabel
       obErrorLog.ThrowError(__FUNCTION__, "AtomRefType called is invalid", obDebug);
       return(false);
     }
-              
+
     return (_atom4refs.size());
   }
 
@@ -1351,7 +1295,7 @@ namespace OpenBabel
     default:
       obErrorLog.ThrowError(__FUNCTION__, "AtomRefType called is invalid", obDebug);
       return(false);
-    }  
+    }
   }
   std::vector<unsigned int> OBChiralData::GetAtom4Refs(atomreftype t) const
   {
@@ -1426,6 +1370,162 @@ namespace OpenBabel
     _atom4refc.clear();
   }
 
+//
+//member functions for OBDOSData class
+//
+
+/*!
+**\brief Assign the data
+**\param fermi The Fermi energy in eV
+**\param vEnergies Energy levels in eV
+**\param vDensities Density of states in (number of states) / (unit cell)
+**\param vIntegration Integrated DOS
+*/
+void OBDOSData::SetData(double fermi,
+                        const std::vector<double> & vEnergies,
+                        const std::vector<double> & vDensities,
+                        const std::vector<double> & vIntegration)
+{
+  this->_fermi = fermi;
+  this->_vEnergies = vEnergies;
+  this->_vIntegration = vIntegration;
+  this->_vDensities = vDensities;
+}
+
+  // member functions for OBOrbitalData
+
+  void OBOrbitalData::LoadClosedShellOrbitals(std::vector<double> energies, std::vector<std::string> symmetries, int alphaHOMO)
+  {
+    if (energies.size() < symmetries.size())
+      return; // something is very weird -- it's OK to pass no symmetries (we'll assume "A")
+    if (energies.size() == 0)
+      return;
+    if (alphaHOMO > energies.size())
+      return;
+
+    _alphaHOMO = alphaHOMO;
+    _alphaOrbitals.clear();
+    _betaHOMO = 0;
+    _betaOrbitals.clear();
+    _openShell = false;
+
+    if (symmetries.size() < energies.size()) // pad with "A" symmetry
+      for (unsigned int i = symmetries.size(); i < energies.size(); ++i)
+        symmetries.push_back("A");
+
+    OBOrbital currentOrbital;
+    for (unsigned int i = 0; i < energies.size(); ++i)
+      {
+        if (i < alphaHOMO)
+          currentOrbital.SetData(energies[i], 2.0, symmetries[i]);
+        else
+          currentOrbital.SetData(energies[i], 0.0, symmetries[i]);
+
+        _alphaOrbitals.push_back(currentOrbital);
+      }
+  }
+
+  void OBOrbitalData::LoadAlphaOrbitals(std::vector<double> energies, std::vector<std::string> symmetries, int alphaHOMO)
+  {
+    if (energies.size() < symmetries.size())
+      return; // something is very weird -- it's OK to pass no symmetries (we'll assume "A")
+    if (energies.size() == 0)
+      return;
+    if (alphaHOMO > energies.size())
+      return;
+
+    _alphaHOMO = alphaHOMO;
+    _alphaOrbitals.clear();
+    _openShell = true;
+
+    if (symmetries.size() < energies.size()) // pad with "A" symmetry
+      for (unsigned int i = symmetries.size(); i < energies.size(); ++i)
+        symmetries.push_back("A");
+
+    OBOrbital currentOrbital;
+    for (unsigned int i = 0; i < energies.size(); ++i)
+      {
+        if (i < alphaHOMO)
+          currentOrbital.SetData(energies[i], 2.0, symmetries[i]);
+        else
+          currentOrbital.SetData(energies[i], 0.0, symmetries[i]);
+
+        _alphaOrbitals.push_back(currentOrbital);
+      }
+  }
+
+  void OBOrbitalData::LoadBetaOrbitals(std::vector<double> energies, std::vector<std::string> symmetries, int betaHOMO)
+  {
+    if (energies.size() < symmetries.size())
+      return; // something is very weird -- it's OK to pass no symmetries (we'll assume "A")
+    if (energies.size() == 0)
+      return;
+    if (betaHOMO > energies.size())
+      return;
+
+    _betaHOMO = betaHOMO;
+    _betaOrbitals.clear();
+    _openShell = true;
+
+    if (symmetries.size() < energies.size()) // pad with "A" symmetry
+      for (unsigned int i = symmetries.size(); i < energies.size(); ++i)
+        symmetries.push_back("A");
+
+    OBOrbital currentOrbital;
+    for (unsigned int i = 0; i < energies.size(); ++i)
+      {
+        if (i < betaHOMO)
+          currentOrbital.SetData(energies[i], 2.0, symmetries[i]);
+        else
+          currentOrbital.SetData(energies[i], 0.0, symmetries[i]);
+
+        _betaOrbitals.push_back(currentOrbital);
+      }
+  }
+
+//
+//member functions for OBElectronicTransitionData class
+//
+
+/*!
+**\brief Assign the basic excitation data
+**\param vWavelengths Wavelengths in nm
+**\param vForces Oscillator strengths
+*/
+void OBElectronicTransitionData::SetData(const std::vector<double> & vWavelengths,
+                                  const std::vector<double> & vForces)
+{
+  this->_vWavelengths = vWavelengths;
+  this->_vForces = vForces;
+}
+
+/*!
+**\brief Assign the electronic dipole strengths
+**\param vEDipole Electronic dipole moment strength
+*/
+void OBElectronicTransitionData::SetEDipole(const std::vector<double> & vEDipole)
+{
+  this->_vEDipole = vEDipole;
+}
+
+/*!
+**\brief Assign the rotatory strengths (velocity)
+**\param vRotatoryStrengthsVelocity Vector containing the rotatory strengths
+*/
+void OBElectronicTransitionData::SetRotatoryStrengthsVelocity(const std::vector<double> & vRotatoryStrengthsVelocity)
+{
+  this->_vRotatoryStrengthsVelocity = vRotatoryStrengthsVelocity;
+}
+
+/*!
+**\brief Assign the rotatory strengths (length)
+**\param vRotatoryStrengthsLength Vector containing the rotatory strengths
+*/
+void OBElectronicTransitionData::SetRotatoryStrengthsLength(const std::vector<double> & vRotatoryStrengthsLength)
+{
+  this->_vRotatoryStrengthsLength = vRotatoryStrengthsLength;
+}
+
 //
 //member functions for OBVibrationData class
 //
@@ -1445,6 +1545,25 @@ void OBVibrationData::SetData(const std::vector< std::vector< vector3 > > & vLx,
   this->_vIntensities = vIntensities;
 }
 
+/*!
+**\brief Assign the data
+**\param vLx Normal modes in 1/sqrt(a.u.)
+**\param vFrequencies Harmonic frequencies in inverse centimeters
+**\param vIntensities Infrared absorption intensities in KM/Mole
+**\param vRamanActivities Raman activities
+*/
+void OBVibrationData::SetData(const std::vector< std::vector< vector3 > > & vLx,
+                              const std::vector<double> & vFrequencies,
+                              const std::vector<double> & vIntensities,
+                              const std::vector<double> & vRamanActivities)
+{
+  this->_vLx = vLx;
+  this->_vFrequencies = vFrequencies;
+  this->_vIntensities = vIntensities;
+  this->_vRamanActivities = vRamanActivities;
+}
+
+
 /*!
 **\brief Get the number of frequencies
 */
