@@ -6,7 +6,7 @@ Some portions Copyright (C) 2001-2006 by Geoffrey R. Hutchison
 Some Portions Copyright (C) 2008 by Marcus D. Hanwell
 
 This file is part of the Open Babel project.
-For more information, see <http://openbabel.sourceforge.net/>
+For more information, see <http://openbabel.org/>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -111,12 +111,12 @@ namespace OpenBabel
     SetZAxis(z);
 
     // Now set the max and mid as well as spacing
-    _xmax = _xmin + _xdim * x.x() + _xdim * y.x() + _xdim * z.x();
-    _ymax = _ymin + _ydim * x.y() + _ydim * y.y() + _ydim * z.y();
-    _zmax = _zmin + _zdim * x.z() + _zdim * y.z() + _zdim * z.z();
-    _spacing = (_xmax - _xmin)/_xdim + (_ymax - _ymin)/_ydim +
-               (_zmax - _zmin)/_zdim;
-    _spacing /= 3.0;
+    _xmax = _xmin + (_xdim-1) * (x.x() + y.x() + z.x());
+    _ymax = _ymin + (_ydim-1) * (x.y() + y.y() + z.y());
+    _zmax = _zmin + (_zdim-1) * (x.z() + y.z() + z.z());
+    // FIXME If we want support for cubes aligned along arbitrary axes then
+    // averaging the x, y and z directions will not work. Just use x for now.
+    _spacing = (_xmax - _xmin)/(_xdim-1);
     _halfSpace= _spacing/2.0;
     _inv_spa=1.0/_spacing;
   }
@@ -347,7 +347,7 @@ namespace OpenBabel
 
   ostream& operator<< ( ostream &os, const  OBFloatGrid& fg)
   {
-    //FIXME: this code stores the data in way that depends on
+    //@todo: this code stores the data in way that depends on
     // the bit representation of floating-point numbers. One can say
     // it's OK because IEEE754 is a widely accepted standard, but then
     // one should at least make it write the data in an endianness-
