@@ -151,6 +151,15 @@ namespace OpenBabel {
       if(conv.SetInFormat("smi") && conv.Read(&mol))
          // ...
       @endcode
+      
+      An alternative way is more convenient if using bindings from another language:
+      @code
+      std::string SmilesString;
+      OBMol mol;
+      OBConversion conv;
+      if(conv.SetInFormat("smi") && conv.ReadString(&mol, SmilesString))
+         // ...
+      @endcode
 
       <b>To do a file conversion without manipulating the molecule.</b>
 
@@ -165,6 +174,27 @@ namespace OpenBabel {
       }
       @endcode
 
+      <b>To read a multi-molecule file if using bindings from another language</b>
+
+      The first molecule should be read using ReadFile, and subsequent molecules using Read,
+      as follows:
+      @code
+      #include <openbabel/obconversion.h> //mol.h is not needed
+      OBConversion conv;
+      OBMol mol;
+      bool success = conv.SetInFormat("sdf");
+      if(success)
+      {
+         bool notatend = conv.ReadFile(&mol, "myfile.sdf");
+         // Do something with mol
+	 while(notatend)
+	 {
+             notatend = conv.Read(&mol);
+	     // Do something with mol
+	 }
+      }
+      @endcode
+
       <b>To add automatic format conversion to an existing program.</b>
 
       The existing program inputs from the file identified by the
@@ -621,7 +651,7 @@ namespace OpenBabel {
         if(Count==(int)EndNumber)
           ReadyToInput=false; //stops any more objects being read
 
-        rInlen = (pInStream->tellg() - rInpos);
+        rInlen = pInStream ? pInStream->tellg() - rInpos : 0;
          // - (pLineEndBuf ? pLineEndBuf->getCorrection() : 0); //correction for CRLF
 
         if(pOb)
@@ -953,9 +983,10 @@ namespace OpenBabel {
 
     ifstream *ifs = new ifstream;
     NeedToFreeInStream = true; // make sure we free this
-    ios_base::openmode imode =
-      pInFormat->Flags() & READBINARY ? ios_base::in|ios_base::binary : ios_base::in;
 
+    ios_base::openmode imode = ios_base::in|ios_base::binary; //now always binary because may be gzipped
+//      pInFormat->Flags() & READBINARY ? ios_base::in|ios_base::binary : ios_base::in;
+      
     ifs->open(filePath.c_str(),imode);
     if(!ifs || !ifs->good())
       {
@@ -1150,6 +1181,7 @@ namespace OpenBabel {
   int OBConversion::FullConvert(std::vector<std::string>& FileList, std::string& OutputFileName,
                                 std::vector<std::string>& OutputFileList)
   {
+    //OBConversion::OutFilename = OutputFileName; ready for 2.4.0
 
     istream* pIs=NULL;
     ostream* pOs=NULL;
@@ -1502,6 +1534,11 @@ Additional options :
 
   void OBConversion::SetOptions(const char* options, Option_type opttyp)
   {
+    if(!*options) // "" clears all
+    {
+      OptionsArray[opttyp].clear();
+      return;
+    }
     while(*options)
       {
         string ch(1, *options++);
