@@ -1613,88 +1613,40 @@ namespace OpenBabel
 
   static int GetChiralFlag(AtomExpr *expr)
   {
-    int size=0;
-#define OB_EVAL_STACKSIZE 40
+    int tmp1,tmp2;
 
-    AtomExpr *stack[OB_EVAL_STACKSIZE];
-    memset(stack,'\0',sizeof(AtomExpr*)*OB_EVAL_STACKSIZE);
-#undef OB_EVAL_STACKSIZE
-
-    bool lftest=true;
-
-    for (size=0,stack[size] = expr;size >= 0;expr=stack[size])
+    switch (expr->type)
       {
-        switch (expr->type)
-          {
-          case AE_LEAF:
-            if (expr->leaf.prop == AL_CHIRAL)
-              return(expr->leaf.value);
-            size--;
-            break;
+        case AE_LEAF:
+          if (expr->leaf.prop == AL_CHIRAL)
+            return expr->leaf.value;
+          break;
 
-          case AE_ANDHI:
-          case AE_ANDLO:
+        case AE_ANDHI:
+        case AE_ANDLO:
+          tmp1 = GetChiralFlag(expr->bin.lft);
+          tmp2 = GetChiralFlag(expr->bin.rgt);
+          if (tmp1 == 0) return tmp2;
+          if (tmp2 == 0) return tmp1;
+          if (tmp1 == tmp2) return tmp1;
+          break;
 
-            if (stack[size+1] == expr->bin.rgt)
-              size--;
-            else if (stack[size+1] == expr->bin.lft)
-              {
-                if (lftest)
-                  {
-                    size++;
-                    stack[size] = expr->bin.rgt;
-                  }
-                else
-                  size--;
-              }
-            else
-              {
-                size++;
-                stack[size] = expr->bin.lft;
-              }
-            break;
+        case AE_OR:
+          tmp1 = GetChiralFlag(expr->bin.lft);
+          tmp2 = GetChiralFlag(expr->bin.rgt);
+          if (tmp1 == 0 || tmp2 == 0) return 0;
+          if (tmp1 == tmp2) return tmp1;
+          break;
 
-          case AE_OR:
-
-            if (stack[size+1] == expr->bin.rgt)
-              size--;
-            else if (stack[size+1] == expr->bin.lft)
-              {
-                if (!lftest)
-                  {
-                    size++;
-                    stack[size] = expr->bin.rgt;
-                  }
-                else
-                  size--;
-              }
-            else
-              {
-                size++;
-                stack[size] = expr->bin.lft;
-              }
-            break;
-
-          case AE_NOT:
-            if (stack[size+1] != expr->mon.arg)
-              {
-                size++;
-                stack[size] = expr->mon.arg;
-              }
-            else
-              {
-                lftest = !lftest;
-                size--;
-              }
-            break;
-
-          case AE_RECUR:
-            size--;
-            break;
-          }
+        case AE_NOT:
+          // Treat [!@] as [@@], and [!@@] as [@]
+          tmp1 = GetChiralFlag(expr->mon.arg);
+          if (tmp1 == AL_ANTICLOCKWISE) return AL_CLOCKWISE;
+          if (tmp1 == AL_CLOCKWISE) return AL_ANTICLOCKWISE;
+          break;
       }
 
-    return((int)false);
+    return 0;
   }
 
   Pattern *OBSmartsPattern::ParseSMARTSPart( Pattern *result, int part )
@@ -2699,10 +2651,11 @@ namespace OpenBabel
             // In other words, ring closure bonds will be handled in the order
             // of their corresponding ring opening. This is important for stereo.
             bs = pat->bond[pat->bond_parse_order[k]];
-            if (bs.dst == j)
-              nbrs.push_back(bs.src);
-            else if (bs.src == j)
-              nbrs.push_back(bs.dst);
+
+          if (bs.dst == j)
+            nbrs.push_back(bs.src);
+          else if (bs.src == j)
+            nbrs.push_back(bs.dst);
 
           if (nbrs.size() < 3)
             continue;
@@ -2738,9 +2691,9 @@ namespace OpenBabel
               smartsConfig.specified = false;
           }
 
-          cout << "smarts config = " << smartsConfig << endl;
-          cout << "molecule config = " << ts->GetConfig() << endl;
-          cout << "match = " << (ts->GetConfig() == smartsConfig) << endl;
+          // cout << "smarts config = " << smartsConfig << endl;
+          // cout << "molecule config = " << ts->GetConfig() << endl;
+          // cout << "match = " << (ts->GetConfig() == smartsConfig) << endl;
 
           // and save the match if the two configurations are the same
           if (ts->GetConfig() != smartsConfig)
