@@ -1,16 +1,16 @@
 /**********************************************************************
 patty.cpp - Programmable atom typer.
- 
+
 Copyright (C) 1998-2001 by OpenEye Scientific Software, Inc.
 Some portions Copyright (C) 2001-2006 by Geoffrey R. Hutchison
- 
+
 This file is part of the Open Babel project.
-For more information, see <http://openbabel.sourceforge.net/>
- 
+For more information, see <http://openbabel.org/>
+
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation version 2 of the License.
- 
+
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -135,7 +135,7 @@ namespace OpenBabel
     char tmp_str[BUFF_SIZE];
     unsigned int i;
     OBSmartsPattern *sp;
-    
+
     for ( i = 0 ; i < rules.size() ; i++ )
       {
         strncpy(buffer, rules[i].c_str(), BUFF_SIZE);
@@ -169,7 +169,7 @@ namespace OpenBabel
         _sp[i]->Match(mol);
         vector<vector<int> > match = _sp[i]->GetMapList();
         //vector<vector<int> >& match = _sp[i]->GetMapList();
-        if (match.size())
+        if (!match.empty())
           {
             if (debug)
               {
@@ -204,7 +204,7 @@ namespace OpenBabel
         _sp[i]->Match(mol);
         vector<vector<int> > match = _sp[i]->GetMapList();
         //vector<vector<int> >& match = _sp[i]->GetMapList();
-        if (match.size())
+        if (!match.empty())
           {
             if (debug)
               {
