@@ -1,16 +1,16 @@
 /**********************************************************************
 ring.cpp - Deal with rings, find smallest set of smallest rings (SSSR).
- 
+
 Copyright (C) 1998-2001 by OpenEye Scientific Software, Inc.
 Some portions Copyright (C) 2001-2006 by Geoffrey R. Hutchison
- 
+
 This file is part of the Open Babel project.
-For more information, see <http://openbabel.sourceforge.net/>
- 
+For more information, see <http://openbabel.org/>
+
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation version 2 of the License.
- 
+
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -26,6 +26,7 @@ using namespace std;
 
 namespace OpenBabel
 {
+  OBRingTyper      ringtyper;
 
   /*! \class OBRing ring.h <openbabel/ring.h>
     \brief Stores information on rings in a molecule from SSSR perception.
@@ -86,6 +87,13 @@ namespace OpenBabel
     obErrorLog.ThrowError(__FUNCTION__,
                           "Ran OpenBabel::FindSSSR", obAuditMsg);
 
+    // Delete any old data before we start finding new rings
+    // The following procedure is slow
+    // So if client code is multi-threaded, we don't want to make them wait
+    if (HasData("SSSR")) {
+      DeleteData("SSSR");
+    }
+
     OBRing *ring;
     vector<OBRing*>::iterator j;
 
@@ -127,15 +135,160 @@ namespace OpenBabel
             //rs.WriteRings();
           }
 
-        if (HasData(OBGenericDataType::RingData))
-          {
-            DeleteData(OBGenericDataType::RingData);
-          }
-        
-        SetData(new OBRingData);
-        OBRingData *rd = (OBRingData*)GetData(OBGenericDataType::RingData);
+        OBRingData *rd = new OBRingData();
         rd->SetOrigin(perceived); // to separate from user or file input
+        rd->SetAttribute("SSSR");
         rd->SetData(vr);
+        SetData(rd);
+      }
+  }
+
+  std::vector<unsigned int> atomRingToBondRing(OBMol *mol, const std::vector<int> &atoms)
+  {
+    std::vector<unsigned int> bonds;
+    for (unsigned int i = 0; i < atoms.size() - 1; ++i) {
+      unsigned int beginIndex = atoms[i];
+      unsigned int endIndex = atoms[i+1];
+      unsigned int index = mol->GetBond(beginIndex, endIndex)->GetIdx();
+      bonds.push_back(index);
+    }
+    bonds.push_back(mol->GetBond(atoms[0], atoms[atoms.size()-1])->GetIdx());
+    return bonds;
+  }
+
+  /**
+   * This function finds the LSSR containing all relevant cycles. A cycle is
+   * relevant if it belongs to at least one minimum cycle basis. Another
+   * description is more useful though:
+   *
+   * A cycle (C) is relevant if:
+   * - no smaller cycles C_i, ..., C_k exist such that C = C_1 + ... + C_k
+   * - both bonds & atoms are checked
+   *
+   * This is based on lemma 1 from:
+   *
+   * P. Vismara, Union of all the minimum cycle bases of a graph, The electronic
+   * journal of combinatorics, Vol. 4, 1997
+   * http://www.emis.de/journals/EJC/Volume_4/PostScriptfiles/v4i1r9.ps
+   */
+  void visitRing(OBMol *mol, OBRing *ring, std::vector<OBRing*> &rlist, std::vector<OBRing*> &rignored)
+  {
+    const std::vector<int> &atoms = ring->_path;
+    OBBitVec mask;
+    // Make sure mask is the same size as the maximum ring atom/bond index.
+    mask.SetBitOn(mol->NumAtoms());
+    mask.SetBitOn(mol->NumBonds());
+
+    //
+    // Remove larger rings that cover the same atoms as smaller rings.
+    //
+    mask.Clear();
+    for (unsigned int j = 0; j < rlist.size(); ++j)
+      // Here we select only smaller rings.
+      if (rlist[j]->_path.size() < ring->_path.size())
+        mask |= rlist[j]->_pathset;
+
+    mask = mask & ring->_pathset;
+
+    bool containsSmallerAtomRing = (mask == ring->_pathset) ? true : false;
+
+    // Translate ring atom indexes to ring bond indexes.
+    std::vector<unsigned int> bonds = atomRingToBondRing(mol, ring->_path);
+    OBBitVec bondset;
+    for (unsigned int i = 0; i < bonds.size(); ++i)
+      bondset.SetBitOn(bonds[i]);
+
+    //
+    // Remove larger rings that cover the same bonds as smaller rings.
+    //
+    mask.Clear();
+    for (unsigned int j = 0; j < rlist.size(); ++j) {
+      std::vector<unsigned int> otherBonds = atomRingToBondRing(mol, rlist[j]->_path);
+      OBBitVec bs;
+      for (unsigned int i = 0; i < otherBonds.size(); ++i)
+        bs.SetBitOn(otherBonds[i]);
+
+      // Here we select only smaller rings.
+      if (otherBonds.size() < bonds.size())
+        mask |= bs;
+    }
+
+    mask = mask & bondset;
+
+    bool containsSmallerBondRing = (mask == bondset) ? true : false;
+
+    // The ring is part of the LSSR if all it's atoms and bonds are not
+    // found in smaller rings.
+    if (!containsSmallerAtomRing || !containsSmallerBondRing) {
+      rlist.push_back(ring);
+    } else {
+      rignored.push_back(ring);
+    }
+  }
+
+
+  void OBMol::FindLSSR()
+  {
+    if (HasLSSRPerceived())
+      return;
+    SetLSSRPerceived();
+    obErrorLog.ThrowError(__FUNCTION__,
+                          "Ran OpenBabel::FindLSSR", obAuditMsg);
+
+    // Delete any old data before we start finding new rings
+    // The following procedure is slow
+    // So if client code is multi-threaded, we don't want to make them wait
+    if (HasData("LSSR")) {
+      DeleteData("LSSR");
+    }
+
+    OBRing *ring;
+    vector<OBRing*>::iterator j;
+
+    //get frerejaque taking int account multiple possible spanning graphs
+    int frj = DetermineFRJ(*this);
+    if (frj)
+      {
+        vector<OBRing*> vr;
+        FindRingAtomsAndBonds();
+
+        OBBond *bond;
+        vector<OBBond*> cbonds;
+        vector<OBBond*>::iterator k;
+
+        //restrict search for rings around closure bonds
+        for (bond = BeginBond(k);bond;bond = NextBond(k))
+          if (bond->IsClosure())
+            cbonds.push_back(bond);
+
+        if (!cbonds.empty())
+          {
+            OBRingSearch rs;
+            //search for all rings about closures
+            vector<OBBond*>::iterator i;
+
+            for (i = cbonds.begin();i != cbonds.end();++i)
+              rs.AddRingFromClosure(*this,(OBBond*)*i);
+
+            rs.SortRings();
+            rs.RemoveRedundant(-1); // -1 means LSSR
+
+            //store the LSSR set
+
+            for (j = rs.BeginRings();j != rs.EndRings();++j)
+              {
+                ring = new OBRing ((*j)->_path,NumAtoms()+1);
+                ring->SetParent(this);
+                vr.push_back(ring);
+              }
+            //rs.WriteRings();
+          }
+
+        OBRingData *rd = new OBRingData();
+        rd->SetOrigin(perceived); // to separate from user or file input
+        rd->SetAttribute("LSSR");
+        rd->SetData(vr);
+        SetData(rd);
       }
   }
 
@@ -174,7 +327,6 @@ namespace OpenBabel
 
   void OBRingSearch::RemoveRedundant(int frj)
   {
-    OBBitVec tmp;
     int i,j;
 
     //remove identical rings
@@ -187,7 +339,28 @@ namespace OpenBabel
             break;
           }
 
+    if (_rlist.size() == 0)
+      return; // nothing to do
+
+    // handle LSSR
+    if (frj < 0) {
+      OBMol *mol = _rlist[0]->GetParent();
+      std::vector<OBRing*> rlist, rignored;
+      for (unsigned int i = 0; i < _rlist.size(); ++i) {
+        visitRing(mol, _rlist[i], rlist, rignored);
+      }
+      for (unsigned int i = 0; i < rignored.size(); ++i)
+        delete rignored[i];
+      _rlist = rlist;
+      return;
+    }
+
+    // exit if we already have frj rings
+    if (_rlist.size() == (unsigned)frj)
+      return;
+
     //make sure tmp is the same size as the rings
+    OBBitVec tmp;
     for (j = 0;j < (signed)_rlist.size();++j)
       tmp = (_rlist[j])->_pathset;
 
@@ -278,6 +451,10 @@ namespace OpenBabel
     for (i = t2.begin();i != t2.end();++i)
       if (*i)
         delete *i;
+
+    // set parent for all rings
+    for (unsigned int j = 0; j < _rlist.size(); ++j)
+      _rlist[j]->SetParent(&mol);
   }
 
   bool OBRingSearch::SaveUniqueRing(deque<int> &d1,deque<int> &d2)
@@ -316,10 +493,10 @@ namespace OpenBabel
     for (i = _rlist.begin();i != _rlist.end();++i)
       delete *i;
   }
-  
+
   bool CompareRingSize(const OBRing *a,const OBRing *b)
   {
-    return(a->Size() < b->Size());
+    return(a->Size() == b->Size() ? a->ring_id < b->ring_id : a->Size() < b->Size()); // ensure stable sort
   }
 
   void OBRingSearch::WriteRings()
@@ -362,7 +539,7 @@ namespace OpenBabel
 
     // don't return if all atoms are visited
     // (For example, some atoms are in multiple rings!) -GRH
-      
+
     if (avisit[natom])
       {
         int j = depth-1;
@@ -405,6 +582,54 @@ namespace OpenBabel
     return(true);
   }
 
+  void OBRing::SetType(char *type)
+  {
+    strncpy(_type,type, sizeof(_type) - 1);
+    _type[sizeof(_type) - 1] = '\0';
+  }
+
+  void OBRing::SetType(std::string &type)
+  {
+     strncpy(_type,type.c_str(), sizeof(_type) - 1);
+    _type[sizeof(_type) - 1] = '\0';
+  }
+
+  char* OBRing::GetType()
+  {
+    OBMol *mol = (OBMol*)GetParent();
+    if (mol && !mol->HasRingTypesPerceived())
+      ringtyper.AssignTypes(*((OBMol*)GetParent()));
+
+    return(_type);
+  }
+
+  unsigned int OBRing::GetRootAtom()
+  {
+    vector<int>::iterator i;
+    OBMol *mol = (OBMol*)GetParent();
+
+    //if (!IsAromatic())
+    //  return 0;
+
+    if (Size() == 6)
+      for (i = _path.begin();i != _path.end();++i)
+        if (!(mol->GetAtom(*i))->IsCarbon())
+	  return (*i);
+
+    if (Size() == 5)
+      for (i = _path.begin();i != _path.end();++i) {
+        OBAtom *atom = mol->GetAtom(*i);
+        if (atom->IsSulfur() && (atom->GetValence() == 2))
+	  return (*i);
+        if (atom->IsOxygen() && (atom->GetValence() == 2))
+	  return (*i);
+        if (atom->IsNitrogen() && (atom->BOSum() == atom->GetValence()))
+	  return (*i);
+      }
+
+    return 0;
+  }
+
   bool OBRing::IsMember(OBAtom *a)
   {
     return(_pathset.BitIsOn(a->GetIdx()));
@@ -419,6 +644,7 @@ namespace OpenBabel
   {
     _pathset.FromVecInt(_path);
     _pathset.Resize(size);
+    _type[0] = '\0';
   }
 
   /*!
@@ -431,6 +657,7 @@ namespace OpenBabel
   {
     //member data
     _parent = src._parent;  //this may not be what you want, but it's a default
+    _type[0] = '\0';
   }
 
   /*!
@@ -450,6 +677,7 @@ namespace OpenBabel
     _path = src._path;
     _pathset = src._pathset;
     _parent = src._parent; //note, this may not be what you want
+    _type[0] = '\0';
 
     return(*this);
   }
@@ -500,7 +728,7 @@ namespace OpenBabel
     _prv = prv;
   }
 
-  //! The supplied path is built up of OBAtom nodes, with the root atom 
+  //! The supplied path is built up of OBAtom nodes, with the root atom
   //! the last item in the vector.
   void OBRTree::PathToRoot(vector<OBAtom*> &path)
   {
