@@ -2,7 +2,7 @@
  inchiwrite.cpp - tests for writing InChIs
 
  This file is part of the Open Babel project.
- For more information, see <http://openbabel.sourceforge.net/>
+ For more information, see <http://openbabel.org/>
 
  Copyright (C) 2007 by Chris Morley
 
@@ -45,6 +45,14 @@ int main(int argc, char* argv[])
     "It can contain other text interspersed." << endl;
     return 1;
   }
+
+  // Define location of file formats for testing
+  #ifdef FORMATDIR
+    char env[BUFF_SIZE];
+    snprintf(env, BUFF_SIZE, "BABEL_LIBDIR=%s", FORMATDIR);
+    putenv(env);
+  #endif
+
   ifstream ifs(argv[1]);
   if(!ifs) { 
     cout << "# Cannot open " << argv[1];
@@ -87,7 +95,7 @@ int main(int argc, char* argv[])
     return 2;
   }
 
-  cout << "1.." << count << " # molecules converted\n";
+  cout << "1.." << count << "\n # molecules converted\n";
 
   //Compare ssout with the correct InChIs
   int nfail=0;
