@@ -5,7 +5,7 @@ Copyright (C) 2003 Fabien Fontaine
 Some portions Copyright (C) 2004-2005 Geoffrey R. Hutchison
  
 This file is part of the Open Babel project.
-For more information, see <http://openbabel.sourceforge.net/>
+For more information, see <http://openbabel.org/>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -28,9 +28,9 @@ GNU General Public License for more details.
 #include <openbabel/obconversion.h>
 #include <openbabel/parsmart.h>
 
-#ifdef MSC_VER
+#ifdef _MSC_VER
 	typedef char TCHAR;
-	#include "XGetOpt.h"
+	#include "getopt.h"
 #else
 	#include <unistd.h>
 #endif
@@ -337,7 +337,7 @@ int main(int argc,char **argv)
 *
 * Open Babel is currently maintained by \b Geoff \b Hutchison, \b Chris \b Morley and \b Michael \b Banck.
 *
-* For more contributors to Open Babel, see http://openbabel.sourceforge.net/THANKS.shtml
+* For more contributors to Open Babel, see http://openbabel.org/THANKS.shtml
 *
 * \par COPYRIGHT
 *  Copyright (C) 1998-2001 by OpenEye Scientific Software, Inc.
@@ -351,6 +351,6 @@ int main(int argc,char **argv)
 *  GNU General Public License for more details.
 *
 * \par SEE ALSO
-*   The web pages for Open Babel can be found at: http://openbabel.sourceforge.net/ \n
+*   The web pages for Open Babel can be found at: http://openbabel.org/ \n
 *   A guide for constructing SMARTS patterns can be found at: http://www.daylight.com/dayhtml/doc/theory/theory.smarts.html
 **/
